#include "J2_OGDR_to_BUFR.h"

int main(int argc, char *argv[])
{
  short int siStatus;
  short int siComLinStatus;
  uint32_t uliTotBUFRLength;
  FILE*    fpBUFROut;
  unsigned char ucPrepReq;
  unsigned short int usiSpaceCraftID = 0;
  unsigned short int usiDay = 0;
  unsigned short int usiHour = 0;
  unsigned short int usiMinute = 0;
  short int siPointInBulletin;
  short int siBulletinSeq;
  char *pzOutputFile;
  unsigned short int usiFOV;
  long int liOGDRSecs;
  char azJ2FileName[BUFSIZ];
  char azProdVers[2];
  char azCycle[4];
  short int siCycle;
  char azPass[4];
  short int siPass;
  char azStopTime[16];
  char *pzSubString;
  int iStrIndex;

/**********************netCDF variables***********/
/* General variables */
  int iNcid, iVarLoop, iRetVal;
  size_t iNcLength, iStrLen;

/* Dimension Ids */
  int iTimeDimId;

/* Variable Ids */
/* AR 63 - Add radiometer state flag to variable ID list (MOD/SSE, 11/06/08) */
/* AR 160 - Add variables not formerly encoded in BUFR (MOD/SSE, 18/01/10) */
  int iTimeVarId;
  int iLatVarId, iLonVarId, iSurfVarId, iAltEchoVarId, iRadSurfVarId;
  int iTb187VarId, iTb238VarId, iTb340VarId;
  int iWSpdUVarId, iWSpdVVarId, iWSpdAltVarId, iWSpdRadVarId, iRadWVVarId, iRadLWVarId;
  int iMeanDynTopVarId, iHeightVarId, iInstAltRVarId, iOffNadPtfVarId, iOffNadWvVarId;
  int iModIonCorVarId, iAltIonCorVarId, iMDryTCorVarId, iMWetTCorVarId, iRadWetTCorVarId;
  int iIntMSSVarId, iTSol1VarId, iTSol2FVarId, iInterMetFVarId, iOrbStatFVarId, iRadStatFVarId;
  int iAltStatFVarId, iAltKuRFVarId, iAltCRFVarId, iAltKuSWHFVarId, iAltCSWHFVarId;
  int iAltKuSigFVarId, iAltCSigFVarId, iOffNadKuFVarId, iOffNafPtfFVarId, iCorKuRFVarId;
  int iCorCRFVarId, iCorKuSWHFVarId, iCorCSWHFVarId, iCorKuSigFVarId, iCorCSigFVarId;
  int iRainFVarId, iQTb187FVarId, iQTb238FVarId, iQTb340FVarId, iTbInterFVarId;
  int iIceVarId, iRangeKuVarId, iRMSRanKuVarId, iNumRanKuVarId, iInstCorrKuVarId;
  int iSSBCorrKuVarId, iSWHKuVarId, iRMSSWHKuVarId, iNumSWHKuVarId, iInstCorrSWHKuVarId;
  int iSig0KuVarId, iRMSSig0KuVarId, iNumSig0KuVarId, iInstCorrSig0KuVarId, iAtCorrSig0KuVarId;
  int iAGCKuVarId, iRMSAGCKuVarId, iNumAGCKuVarId, iRangeCVarId, iRMSRanCVarId;
  int iNumRanCVarId, iInstCorrCVarId, iSSBCorrCVarId, iSWHCVarId, iRMSSWHCVarId;
  int iNumSWHCVarId, iInstCorrSWHCVarId, iSig0CVarId, iRMSSig0CVarId, iNumSig0CVarId;
  int iInstCorrSig0CVarId, iAtCorrSig0CVarId, iAGCCVarId, iRMSAGCCVarId, iNumAGCCVarId;
  int iMSSVarId, iMSSAltVarId, iGeoidVarId, iBathyVarId, iSolidTideVarId;
  int iGeoTide1VarId, iGeoTide2VarId, iLoadTide1VarId, iLoadTide2VarId, iOcTideEqVarId;
  int iOcTideNonEqVarId, iPoleTideVarId, iInvBathVarId, iHfFlucVarId;
  int iASFCBandVarId, iASFCBandStatVarId, iASFKuBandStatVarId, iASFBandSeqVarId;
  int iECMWFMapAvlVarId, iInterpFlagMDTVarId, iSSHAVarId;

/* Variable values */
/* AR 63 - Add radiometer state flag to variable list (MOD/SSE, 11/06/08) */
/* AR 160 - Add variables not formerly encoded in BUFR (MOD/SSE, 18/01/10) */
  signed char *pscSurfVals, *pscAltEchoVals, *pscRadSurfVals, *pscIntMSSVals;
  signed char *pscTSol1FVals, *pscTSol2FVals, *pscInterMetFVals; 
  signed char *pscOrbStatFVals, *pscRadStatFVals;
  signed char *pscAltStatFVals, *pscAltKuRFVals, *pscAltCRFVals, *pscAltKuSWHFVals;
  signed char *pscAltCSWHFVals, *pscAltKuSigFVals, *pscAltCSigFVals, *pscOffNadKuFVals;
  signed char *pscOffNafPtfFVals, *pscCorKuRFVals, *pscCorCRFVals, *pscCorKuSWHFVals;
  signed char *pscCorCSWHFVals, *pscCorKuSigFVals, *pscCorCSigFVals, *pscRainFVals;
  signed char *pscQTb187FVals, *pscQTb238FVals, *pscQTb340FVals, *pscTbInterFVals;
  signed char *pscIceVals, *pscNumRanKuVals, *pscNumSWHKuVals, *pscNumSig0KuVals;
  signed char *pscAtCorrSig0KuVals, *pscNumAGCKuVals, *pscNumRanCVals, *pscNumSWHCVals;
  signed char *pscNumSig0CVals, *pscAtCorrSig0CVals, *pscNumAGCCVals;
  signed char *pscASFCBandVals, *pscASFCBandStatVals, *pscASFKuBandStatVals;
  signed char *pscASFBandSeqVals, *pscECMWFMapAvlVals, *pscInterpFlagMDTVals;
  short int *psiWSpdUVals, *psiWSpdVVals, *psiWSpdAltVals, *psiWSpdRadVals;
  short int *psiRadWVVals, *psiRadLWVals, *psiInstAltRVals, *psiOffNadPtfVals; 
  short int *psiOffNadWvVals, *psiModIonCorVals, *psiAltIonCorVals, *psiMDryTCorVals; 
  short int *psiMWetTCorVals, *psiRadWetTCorVals; 
  short int *psiRMSRanKuVals, *psiSSBCorrKuVals, *psiSWHKuVals, *psiRMSSWHKuVals;
  short int *psiInstCorrSWHKuVals, *psiSig0KuVals, *psiRMSSig0KuVals, *psiInstCorrSig0KuVals;
  short int *psiAGCKuVals, *psiRMSAGCKuVals;
  short int *psiRMSRanCVals, *psiSSBCorrCVals, *psiSWHCVals, *psiRMSSWHCVals;
  short int *psiInstCorrSWHCVals, *psiSig0CVals, *psiRMSSig0CVals, *psiInstCorrSig0CVals;
  short int *psiAGCCVals, *psiRMSAGCCVals;
  short int *psiSolidTideVals, *psiLoadTide1Vals, *psiLoadTide2Vals, *psiOcTideEqVals;
  short int *psiOcTideNonEqVals, *psiPoleTideVals, *psiInvBathVals, *psiHfFlucVals; 
  short int *psiTb187Vals, *psiTb238Vals, *psiTb340Vals, *psiSSHAVals;
  int *piLatVals, *piLonVals, *piMeanDynTopVals, *piHeightVals;
  int *piRangeKuVals, *piInstCorrKuVals, *piRangeCVals, *piInstCorrCVals;
  int *piMSSVals, *piMSSAltVals, *piGeoidVals;
  int *piBathyVals, *piGeoTide1Vals, *piGeoTide2Vals;
  double *pdTimeVals;

/* Global attributes */
  char *pzProcCen, *pzCreatTime, *pzAcqStn;
  int iCycleNo, iAbsRevNo;
/***************** End of netCDF variables *******/

/* Parse input arguments for command line options */
  siComLinStatus = siCommandLineOptions(&argc, argv);
  if (siComLinStatus < 0)
  {
    if (OUT)(void)printf("siCommandLineOptions failed with status: %d.\n", siComLinStatus);
    /* Write syslog message for monitoring */
    if (LOG) syslog(LOG_ERR|LOG_LOCAL2, "LOG_ERR: siCommandLineOptions failed with status: %d",
       siComLinStatus);
    return -100;
  }
  else if (siComLinStatus > 0)
  {
    if (OUT)(void)printf("Command line option parsed: %d.\n", siComLinStatus);
    *++argv;
    if (2 == siComLinStatus)      /* fist-version */
    {
      (void)printf("FIST version: ");
      (void)printf("%5.1f\n", FIST_VERS);
      return 0;
    }
    else if (3 == siComLinStatus) /* bufr-table-versions */
    {
      (void)printf("BUFR table version: ");
      (void)printf("%3d.%1.1d\n", BUFR_MT_VERSION, BUFR_LT_VERSION);
      return 0;
    }
    else if (4 == siComLinStatus) /* ecmwf-library-version */
    {
      (void)printf("ECMWF library version: ");
      (void)printf("%4d\n", ECMWF_LIB_VERS);
      return 0;
    }
  }
  else /* siComLinStatus == 0 */
  {
    if (OUT)(void)printf("No command line arguments\n");
  }

/* Parse input arguments for filename parameters */
  siStatus = siParseInputFileName(argc, argv, azJ2FileName, azProdVers,
    azCycle, &siCycle, azPass, &siPass, azStopTime);
  if (siStatus != 0)
  {
    if (OUT)(void)printf("siParseInputFileName failed with status: %d.\n", siStatus);
    /* Write syslog message for monitoring */
    if (LOG) syslog(LOG_ERR|LOG_LOCAL2, "LOG_ERR: siParseInputFileName failed with status: %d",
       siStatus);
    return -101;
  }

/* Write syslog message for monitoring */
  if (LOG)
        syslog(LOG_INFO|LOG_LOCAL2, "LOG_INFO: NetCDF file name: %s", azJ2FileName);

/**********************netCDF code****************/

  /* Open the file. NC_NOWRITE tells netCDF we want read-only access
   * to the file.*/
  if (iRetVal = nc_open(azJ2FileName, NC_NOWRITE, &iNcid))
    ERR(iRetVal);

  /* Inquire about dimension time to get dimid */
  if (iRetVal = nc_inq_dimid(iNcid, "time", &iTimeDimId))
    ERR(iRetVal);

  /* Inquire about dimension time to get length */
  if (iRetVal = nc_inq_dimlen(iNcid, iTimeDimId, &iNcLength))
    ERR(iRetVal);

  /* Inquire about varid of variable "time" */
  if (iRetVal = nc_inq_varid(iNcid, "time", &iTimeVarId))
    ERR(iRetVal);

  /* Inquire about varid of variable "lat" */
  if (iRetVal = nc_inq_varid(iNcid, "lat", &iLatVarId))
    ERR(iRetVal);

  /* Inquire about varid of variable "lon" */
  if (iRetVal = nc_inq_varid(iNcid, "lon", &iLonVarId))
    ERR(iRetVal);

  /* Inquire about variable surface_type */
  if (iRetVal = nc_inq_varid(iNcid, "surface_type", &iSurfVarId))
    ERR(iRetVal);

  /* Inquire about variable alt_echo_type */
  if (iRetVal = nc_inq_varid(iNcid, "alt_echo_type", &iAltEchoVarId))
    ERR(iRetVal);

  /* Inquire about variable rad_surf_type */
  if (iRetVal = nc_inq_varid(iNcid, "rad_surf_type", &iRadSurfVarId))
    ERR(iRetVal);
  /* Inquire about flag variables */
/* ECP 5 - revised variable name in v2.3 of product spec (MOD/SSE, 06/11/08) */
  if (iRetVal = nc_inq_varid(iNcid, "interp_flag_mean_sea_surface", &iIntMSSVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "interp_flag_ocean_tide_sol1", &iTSol1VarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "interp_flag_ocean_tide_sol2", &iTSol2FVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "interp_flag_meteo", &iInterMetFVarId))
    ERR(iRetVal);
/* AR 63 - Use orb_state_flag_diode rather than orb_state_flag_rest (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_varid(iNcid, "orb_state_flag_diode", &iOrbStatFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "alt_state_flag_oper", &iAltStatFVarId))
    ERR(iRetVal);
/* AR 63 - radiometer state flag is available in real JASON2 data.  If its */
/*         not available use altimeter state flag instead as it is similar enough */
/*         (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_varid(iNcid, "rad_state_flag_oper", &iRadStatFVarId))
    iRadStatFVarId = iAltStatFVarId;
/* AR 63 - variable names changed in prod specn v2.1 (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_varid(iNcid, "qual_alt_1hz_range_ku", &iAltKuRFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_alt_1hz_range_c", &iAltCRFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_alt_1hz_swh_ku", &iAltKuSWHFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_alt_1hz_swh_c", &iAltCSWHFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_alt_1hz_sig0_ku", &iAltKuSigFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_alt_1hz_sig0_c", &iAltCSigFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_alt_1hz_off_nadir_angle_wf_ku", &iOffNadKuFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_alt_1hz_off_nadir_angle_pf", &iOffNafPtfFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_inst_corr_1hz_range_ku", &iCorKuRFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_inst_corr_1hz_range_c", &iCorCRFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_inst_corr_1hz_swh_ku", &iCorKuSWHFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_inst_corr_1hz_swh_c", &iCorCSWHFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_inst_corr_1hz_sig0_ku", &iCorKuSigFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_inst_corr_1hz_sig0_c", &iCorCSigFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "rain_flag", &iRainFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_rad_1hz_tb187", &iQTb187FVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_rad_1hz_tb238", &iQTb238FVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "qual_rad_1hz_tb340", &iQTb340FVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "interp_flag_tb", &iTbInterFVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "ice_flag", &iIceVarId))
    ERR(iRetVal);

  /* Inquire about Ku band altimeter variables */
/* AR 63 - variable names changed in prod specn v2.1 (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_varid(iNcid, "range_ku", &iRangeKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "range_rms_ku", &iRMSRanKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "range_numval_ku", &iNumRanKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "net_instr_corr_range_ku", &iInstCorrKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "sea_state_bias_ku", &iSSBCorrKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "swh_ku", &iSWHKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "swh_rms_ku", &iRMSSWHKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "swh_numval_ku", &iNumSWHKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "net_instr_corr_swh_ku", &iInstCorrSWHKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "sig0_ku", &iSig0KuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "sig0_rms_ku", &iRMSSig0KuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "sig0_numval_ku", &iNumSig0KuVarId))
    ERR(iRetVal);
/* ECP 5 - revised variable name in v2.3 of product spec (MOD/SSE, 06/11/08) */
  if (iRetVal = nc_inq_varid(iNcid, "net_instr_corr_sig0_ku", &iInstCorrSig0KuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "atmos_corr_sig0_ku", &iAtCorrSig0KuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "agc_ku", &iAGCKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "agc_rms_ku", &iRMSAGCKuVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "agc_numval_ku", &iNumAGCKuVarId))
    ERR(iRetVal);

  /* Inquire about C band altimeter variables */
/* AR 63 - variable names changed in prod specn v2.1 (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_varid(iNcid, "range_c", &iRangeCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "range_rms_c", &iRMSRanCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "range_numval_c", &iNumRanCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "net_instr_corr_range_c", &iInstCorrCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "sea_state_bias_c", &iSSBCorrCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "swh_c", &iSWHCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "swh_rms_c", &iRMSSWHCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "swh_numval_c", &iNumSWHCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "net_instr_corr_swh_c", &iInstCorrSWHCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "sig0_c", &iSig0CVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "sig0_rms_c", &iRMSSig0CVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "sig0_numval_c", &iNumSig0CVarId))
    ERR(iRetVal);
/* ECP 5 - revised variable name in v2.3 of product spec (MOD/SSE, 06/11/08) */
  if (iRetVal = nc_inq_varid(iNcid, "net_instr_corr_sig0_c", &iInstCorrSig0CVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "atmos_corr_sig0_c", &iAtCorrSig0CVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "agc_c", &iAGCCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "agc_rms_c", &iRMSAGCCVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "agc_numval_c", &iNumAGCCVarId))
    ERR(iRetVal);

  /* Inquire about variable tb_187 */
  if (iRetVal = nc_inq_varid(iNcid, "tb_187", &iTb187VarId))
    ERR(iRetVal);

  /* Inquire about variable tb_238 */
  if (iRetVal = nc_inq_varid(iNcid, "tb_238", &iTb238VarId))
    ERR(iRetVal);

  /* Inquire about variable tb_340 */
  if (iRetVal = nc_inq_varid(iNcid, "tb_340", &iTb340VarId))
    ERR(iRetVal);

  /* Inquire about variable wind_speed_model_u */
  if (iRetVal = nc_inq_varid(iNcid, "wind_speed_model_u", &iWSpdUVarId))
    ERR(iRetVal);

  /* Inquire about variable wind_speed_model_v */
  if (iRetVal = nc_inq_varid(iNcid, "wind_speed_model_v", &iWSpdVVarId))
    ERR(iRetVal);

  /* Inquire about variable wind_speed_alt */
  if (iRetVal = nc_inq_varid(iNcid, "wind_speed_alt", &iWSpdAltVarId))
    ERR(iRetVal);

  /* Inquire about variable wind_speed_rad */
  if (iRetVal = nc_inq_varid(iNcid, "wind_speed_rad", &iWSpdRadVarId))
    ERR(iRetVal);

  /* Inquire about variable rad_water_vapor */
  if (iRetVal = nc_inq_varid(iNcid, "rad_water_vapor", &iRadWVVarId))
    ERR(iRetVal);

  /* Inquire about variable rad_liquid_water */
  if (iRetVal = nc_inq_varid(iNcid, "rad_liquid_water", &iRadLWVarId))
    ERR(iRetVal);

  /* Inquire about variable mean_topography */
  if (iRetVal = nc_inq_varid(iNcid, "mean_topography", &iMeanDynTopVarId))
    ERR(iRetVal);

  /* Inquire about variable alt */
  if (iRetVal = nc_inq_varid(iNcid, "alt", &iHeightVarId))
    ERR(iRetVal);

  /* Inquire about variable orb_alt_rate */
  if (iRetVal = nc_inq_varid(iNcid, "orb_alt_rate", &iInstAltRVarId))
    ERR(iRetVal);

  /* Inquire about variable off_nadir_angle_ptf */
/* AR 63 - variable name changed in prod specn v2.1 (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_varid(iNcid, "off_nadir_angle_pf", &iOffNadPtfVarId))
    ERR(iRetVal);

  /* Inquire about variable off_nadir_angle_Ku_wvf */
/* AR 63 - variable name changed in prod specn v2.1 (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_varid(iNcid, "off_nadir_angle_wf_ku", &iOffNadWvVarId))
    ERR(iRetVal);

  /* Inquire about variable iono_corr_GIM_Ku */
/* AR 63 - variable name changed in prod specn v2.1 (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_varid(iNcid, "iono_corr_gim_ku", &iModIonCorVarId))
    ERR(iRetVal);

  /* Inquire about variable iono_corr_alt_Ku */
/* AR 63 - variable name changed in prod specn v2.1 (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_varid(iNcid, "iono_corr_alt_ku", &iAltIonCorVarId))
    ERR(iRetVal);

  /* Inquire about variable model_dry_tropo_corr */
  if (iRetVal = nc_inq_varid(iNcid, "model_dry_tropo_corr", &iMDryTCorVarId))
    ERR(iRetVal);

  /* Inquire about variable model_wet_tropo_corr */
  if (iRetVal = nc_inq_varid(iNcid, "model_wet_tropo_corr", &iMWetTCorVarId))
    ERR(iRetVal);

  /* Inquire about variable rad_wet_tropo_corr */
  if (iRetVal = nc_inq_varid(iNcid, "rad_wet_tropo_corr", &iRadWetTCorVarId))
    ERR(iRetVal);

  /* Inquire about geophysical parameters */
/* ECP 5 - revised variable name in v2.3 of product spec (MOD/SSE, 06/11/08) */
  if (iRetVal = nc_inq_varid(iNcid, "mean_sea_surface", &iMSSVarId))
    ERR(iRetVal);
/* AR 63 - variable mss_tp_along_track not available in J2 O/IGDRs (MOD/SSE, 11/06/08) */
/*
  if (iRetVal = nc_inq_varid(iNcid, "mss_tp_along_track", &iMSSAltVarId))
    ERR(iRetVal);
*/
  if (iRetVal = nc_inq_varid(iNcid, "geoid", &iGeoidVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "bathymetry", &iBathyVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "solid_earth_tide", &iSolidTideVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "ocean_tide_sol1", &iGeoTide1VarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "ocean_tide_sol2", &iGeoTide2VarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "load_tide_sol1", &iLoadTide1VarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "load_tide_sol2", &iLoadTide2VarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "ocean_tide_equil", &iOcTideEqVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "ocean_tide_non_equil", &iOcTideNonEqVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "pole_tide", &iPoleTideVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "inv_bar_corr", &iInvBathVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "hf_fluctuations_corr", &iHfFlucVarId))
    ERR(iRetVal);

/* AR 160 - Access variables not formerly encoded in BUFR (MOD/SSE, 18/01/10) */
  if (iRetVal = nc_inq_varid(iNcid, "alt_state_flag_c_band", &iASFCBandVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "alt_state_flag_c_band_status", &iASFCBandStatVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "alt_state_flag_ku_band_status", &iASFKuBandStatVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "alt_state_flag_band_seq", &iASFBandSeqVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "ecmwf_meteo_map_avail", &iECMWFMapAvlVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "interp_flag_mdt", &iInterpFlagMDTVarId))
    ERR(iRetVal);
  if (iRetVal = nc_inq_varid(iNcid, "ssha", &iSSHAVarId))
    ERR(iRetVal);

/* Allocate memory for time */
  pdTimeVals = (double *)malloc(iNcLength * sizeof(double));

/* Allocate memory for lat, lon and surface */
  piLatVals = (int *)malloc(iNcLength * sizeof(int));
  piLonVals = (int *)malloc(iNcLength * sizeof(int));
  pscSurfVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscAltEchoVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscRadSurfVals = (signed char *)malloc(iNcLength * sizeof(signed char));

/* Allocate memory for flags */
  pscIntMSSVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscTSol1FVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscTSol2FVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscInterMetFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscOrbStatFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
/* AR 63 - use radiometer state flag, which is available in real JASON2 data */
/*         (MOD/SSE, 11/06/08) */
  pscRadStatFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscAltStatFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscAltKuRFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscAltCRFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscAltKuSWHFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscAltCSWHFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscAltKuSigFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscAltCSigFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscOffNadKuFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscOffNafPtfFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscCorKuRFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscCorCRFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscCorKuSWHFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscCorCSWHFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscCorKuSigFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscCorCSigFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscRainFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscQTb187FVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscQTb238FVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscQTb340FVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscTbInterFVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscIceVals = (signed char *)malloc(iNcLength * sizeof(signed char));

/* Allocate memory for Ku band altimeter data */
  piRangeKuVals = (int *)malloc(iNcLength * sizeof(int));
  psiRMSRanKuVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscNumRanKuVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  piInstCorrKuVals = (int *)malloc(iNcLength * sizeof(int));
  psiSSBCorrKuVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiSWHKuVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiRMSSWHKuVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscNumSWHKuVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  psiInstCorrSWHKuVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiSig0KuVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiRMSSig0KuVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscNumSig0KuVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  psiInstCorrSig0KuVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscAtCorrSig0KuVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  psiAGCKuVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiRMSAGCKuVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscNumAGCKuVals = (signed char *)malloc(iNcLength * sizeof(signed char));

/* Allocate memory for C band altimeter data */
  piRangeCVals = (int *)malloc(iNcLength * sizeof(int));
  psiRMSRanCVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscNumRanCVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  piInstCorrCVals = (int *)malloc(iNcLength * sizeof(int));
  psiSSBCorrCVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiSWHCVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiRMSSWHCVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscNumSWHCVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  psiInstCorrSWHCVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiSig0CVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiRMSSig0CVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscNumSig0CVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  psiInstCorrSig0CVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscAtCorrSig0CVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  psiAGCCVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiRMSAGCCVals = (short int *)malloc(iNcLength * sizeof(short int));
  pscNumAGCCVals = (signed char *)malloc(iNcLength * sizeof(signed char));

/* Allocate memory for brightness temperatures */
  psiTb187Vals = (short int *)malloc(iNcLength * sizeof(short int));
  psiTb238Vals = (short int *)malloc(iNcLength * sizeof(short int));
  psiTb340Vals = (short int *)malloc(iNcLength * sizeof(short int));

/* Allocate memory for environmental params */
  psiWSpdUVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiWSpdVVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiWSpdAltVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiWSpdRadVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiRadWVVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiRadLWVals = (short int *)malloc(iNcLength * sizeof(short int));

/* Allocate memory for topography */
  piMeanDynTopVals = (int *)malloc(iNcLength * sizeof(int));
  piHeightVals = (int *)malloc(iNcLength * sizeof(int));
  psiInstAltRVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiOffNadPtfVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiOffNadWvVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiModIonCorVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiAltIonCorVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiMDryTCorVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiMWetTCorVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiRadWetTCorVals = (short int *)malloc(iNcLength * sizeof(short int));

/* Allocate memory for geophysical parameters */
  piMSSVals = (int *)malloc(iNcLength * sizeof(int));
/* AR 63 - variable mss_tp_along_track not available in J2 O/IGDRs (MOD/SSE, 11/06/08) */
/*
  piMSSAltVals = (int *)malloc(iNcLength * sizeof(int));
*/
  piGeoidVals = (int *)malloc(iNcLength * sizeof(int));
  piBathyVals = (int *)malloc(iNcLength * sizeof(int));
  psiSolidTideVals = (short int *)malloc(iNcLength * sizeof(short int));
  piGeoTide1Vals = (int *)malloc(iNcLength * sizeof(int));
  piGeoTide2Vals = (int *)malloc(iNcLength * sizeof(int));
  psiLoadTide1Vals = (short int *)malloc(iNcLength * sizeof(short int));
  psiLoadTide2Vals = (short int *)malloc(iNcLength * sizeof(short int));
  psiOcTideEqVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiOcTideNonEqVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiPoleTideVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiInvBathVals = (short int *)malloc(iNcLength * sizeof(short int));
  psiHfFlucVals = (short int *)malloc(iNcLength * sizeof(short int));

/* AR 160 - Allocate memory for additional parameters parameters (MOD/SSE, 18/01/10) */
  pscASFCBandVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscASFCBandStatVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscASFKuBandStatVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscASFBandSeqVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscECMWFMapAvlVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  pscInterpFlagMDTVals = (signed char *)malloc(iNcLength * sizeof(signed char));
  psiSSHAVals = (short int *)malloc(iNcLength * sizeof(short int));

/* read time values */
  if (iRetVal = nc_get_var_double(iNcid, iTimeVarId, pdTimeVals))
    ERR(iRetVal);

/* read lat, lon and surface values */
  if (iRetVal = nc_get_var_int(iNcid, iLatVarId, piLatVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_int(iNcid, iLonVarId, piLonVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iSurfVarId, pscSurfVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAltEchoVarId, pscAltEchoVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iRadSurfVarId, pscRadSurfVals))
    ERR(iRetVal);

 /* read flags */
  if (iRetVal = nc_get_var_schar(iNcid, iIntMSSVarId, pscIntMSSVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iTSol1VarId, pscTSol1FVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iTSol2FVarId, pscTSol2FVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iInterMetFVarId, pscInterMetFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iOrbStatFVarId, pscOrbStatFVals))
    ERR(iRetVal);
/* AR 63 - read radiometer state flag, which is available in real JASON2 data */
/*         (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_get_var_schar(iNcid, iRadStatFVarId, pscRadStatFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAltStatFVarId, pscAltStatFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAltKuRFVarId, pscAltKuRFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAltCRFVarId, pscAltCRFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAltKuSWHFVarId, pscAltKuSWHFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAltCSWHFVarId, pscAltCSWHFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAltKuSigFVarId, pscAltKuSigFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAltCSigFVarId, pscAltCSigFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iOffNadKuFVarId, pscOffNadKuFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iOffNafPtfFVarId, pscOffNafPtfFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iCorKuRFVarId, pscCorKuRFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iCorCRFVarId, pscCorCRFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iCorKuSWHFVarId, pscCorKuSWHFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iCorCSWHFVarId, pscCorCSWHFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iCorKuSigFVarId, pscCorKuSigFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iCorCSigFVarId, pscCorCSigFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iRainFVarId, pscRainFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iQTb187FVarId, pscQTb187FVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iQTb238FVarId, pscQTb238FVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iQTb340FVarId, pscQTb340FVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iTbInterFVarId, pscTbInterFVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iIceVarId, pscIceVals))
    ERR(iRetVal);

 /* read brightness temperature values */
  if (iRetVal = nc_get_var_short(iNcid, iTb187VarId, psiTb187Vals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iTb238VarId, psiTb238Vals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iTb340VarId, psiTb340Vals))
    ERR(iRetVal);

 /* read Ku band altimeter values */
  if (iRetVal = nc_get_var_int(iNcid, iRangeKuVarId, piRangeKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRMSRanKuVarId, psiRMSRanKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iNumRanKuVarId, pscNumRanKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_int(iNcid, iInstCorrKuVarId, piInstCorrKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iSSBCorrKuVarId, psiSSBCorrKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iSWHKuVarId, psiSWHKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRMSSWHKuVarId, psiRMSSWHKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iNumSWHKuVarId, pscNumSWHKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iInstCorrSWHKuVarId, psiInstCorrSWHKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iSig0KuVarId, psiSig0KuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRMSSig0KuVarId, psiRMSSig0KuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iNumSig0KuVarId, pscNumSig0KuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iInstCorrSig0KuVarId, psiInstCorrSig0KuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAtCorrSig0KuVarId, pscAtCorrSig0KuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iAGCKuVarId, psiAGCKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRMSAGCKuVarId, psiRMSAGCKuVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iNumAGCKuVarId, pscNumAGCKuVals))
    ERR(iRetVal);

 /* read C band altimeter values */
  if (iRetVal = nc_get_var_int(iNcid, iRangeCVarId, piRangeCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRMSRanCVarId, psiRMSRanCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iNumRanCVarId, pscNumRanCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_int(iNcid, iInstCorrCVarId, piInstCorrCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iSSBCorrCVarId, psiSSBCorrCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iSWHCVarId, psiSWHCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRMSSWHCVarId, psiRMSSWHCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iNumSWHCVarId, pscNumSWHCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iInstCorrSWHCVarId, psiInstCorrSWHCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iSig0CVarId, psiSig0CVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRMSSig0CVarId, psiRMSSig0CVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iNumSig0CVarId, pscNumSig0CVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iInstCorrSig0CVarId, psiInstCorrSig0CVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iAtCorrSig0CVarId, pscAtCorrSig0CVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iAGCCVarId, psiAGCCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRMSAGCCVarId, psiRMSAGCCVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iNumAGCCVarId, pscNumAGCCVals))
    ERR(iRetVal);

 /* read environmental params */
  if (iRetVal = nc_get_var_short(iNcid, iWSpdUVarId, psiWSpdUVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iWSpdVVarId, psiWSpdVVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iWSpdAltVarId, psiWSpdAltVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iWSpdRadVarId, psiWSpdRadVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRadWVVarId, psiRadWVVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRadLWVarId, psiRadLWVals))
    ERR(iRetVal);

 /* read topography */
  if (iRetVal = nc_get_var_int(iNcid, iMeanDynTopVarId, piMeanDynTopVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_int(iNcid, iHeightVarId, piHeightVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iInstAltRVarId, psiInstAltRVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iOffNadPtfVarId, psiOffNadPtfVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iOffNadWvVarId, psiOffNadWvVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iModIonCorVarId, psiModIonCorVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iAltIonCorVarId, psiAltIonCorVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iMDryTCorVarId, psiMDryTCorVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iMWetTCorVarId, psiMWetTCorVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iRadWetTCorVarId, psiRadWetTCorVals))
    ERR(iRetVal);

 /* read geophysical parameters */
  if (iRetVal = nc_get_var_int(iNcid, iMSSVarId, piMSSVals))
    ERR(iRetVal);
/* AR 63 - variable mss_tp_along_track not available in J2 O/IGDRs (MOD/SSE, 11/06/08) */
/*
  if (iRetVal = nc_get_var_int(iNcid, iMSSAltVarId, piMSSAltVals))
    ERR(iRetVal);
*/
  if (iRetVal = nc_get_var_int(iNcid, iGeoidVarId, piGeoidVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_int(iNcid, iBathyVarId, piBathyVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iSolidTideVarId, psiSolidTideVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_int(iNcid, iGeoTide1VarId, piGeoTide1Vals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_int(iNcid, iGeoTide2VarId, piGeoTide2Vals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iLoadTide1VarId, psiLoadTide1Vals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iLoadTide2VarId, psiLoadTide2Vals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iOcTideEqVarId, psiOcTideEqVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iOcTideNonEqVarId, psiOcTideNonEqVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iPoleTideVarId, psiPoleTideVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iInvBathVarId, psiInvBathVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iHfFlucVarId, psiHfFlucVals))
    ERR(iRetVal);

/* AR 160 - Allocate memory for additional parameters parameters (MOD/SSE, 18/01/10) */
  if (iRetVal = nc_get_var_schar(iNcid, iASFCBandVarId, pscASFCBandVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iASFCBandStatVarId, pscASFCBandStatVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iASFKuBandStatVarId, pscASFKuBandStatVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iASFBandSeqVarId, pscASFBandSeqVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iECMWFMapAvlVarId, pscECMWFMapAvlVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_schar(iNcid, iInterpFlagMDTVarId, pscInterpFlagMDTVals))
    ERR(iRetVal);
  if (iRetVal = nc_get_var_short(iNcid, iSSHAVarId, psiSSHAVals))
    ERR(iRetVal);

/**********
  for (iVarLoop = 0; iVarLoop < iNcLength; iVarLoop++)
  {
    if (OUT) (void)printf("%d %6.2f %6.2f %f %f %d %f %f %f %f %d %f %f %f %d %f %f %f %f %d\n",
      iVarLoop,
      piLonVals[iVarLoop] / 1.0e6,
      piLatVals[iVarLoop] / 1.0e6,
      (piRangeCVals[iVarLoop] / 1.0e4) + 1.3e6,
      psiRMSRanCVals[iVarLoop] / 1.0e4,
      pscNumRanCVals[iVarLoop],
      piInstCorrCVals[iVarLoop] / 1.0e4,
      psiSSBCorrCVals[iVarLoop] / 1.0e4,
      psiSWHCVals[iVarLoop] / 1.0e3,
      psiRMSSWHCVals[iVarLoop] / 1.0e3,
      pscNumSWHCVals[iVarLoop],
      psiInstCorrSWHCVals[iVarLoop] / 1.0e3,
      psiSig0CVals[iVarLoop] / 1.0e2,
      psiRMSSig0CVals[iVarLoop] / 1.0e2,
      pscNumSig0CVals[iVarLoop],
      psiInstCorrSig0CVals[iVarLoop] / 1.0e2,
      pscAtCorrSig0CVals[iVarLoop] / 1.0e2,
      psiAGCCVals[iVarLoop] / 1.0e2,
      psiRMSAGCCVals[iVarLoop] / 1.0e2,
      pscNumAGCCVals[iVarLoop]);
  }
**********/

/* Get processing centre from global attributes */
  if (iRetVal = nc_inq_attlen(iNcid, NC_GLOBAL, "processing_center", &iStrLen))
    ERR(iRetVal);
  pzProcCen = (char *)malloc(iStrLen + 1);
  if (iRetVal = nc_get_att_text(iNcid, NC_GLOBAL, "processing_center", pzProcCen))
    ERR(iRetVal);
  pzProcCen[iStrLen] = '\0';
  if (OUT) (void)printf("Processing centre: %s\n", pzProcCen);

/* Get product creation time from "history" global attributes */
/* AR 63 - creation time to be extracted from "history" (MOD/SSE, 11/06/08) */
  if (iRetVal = nc_inq_attlen(iNcid, NC_GLOBAL, "history", &iStrLen))
    ERR(iRetVal);
  pzCreatTime = (char *)malloc(iStrLen + 1);
  if (iRetVal = nc_get_att_text(iNcid, NC_GLOBAL, "history", pzCreatTime))
    ERR(iRetVal);
  pzCreatTime[iStrLen] = '\0';
  if (OUT) (void)printf("Product history: %s\n", pzCreatTime);

  /* Get acquisition station name from global attributes */
  if (iRetVal = nc_inq_attlen(iNcid, NC_GLOBAL, "acq_station_name", &iStrLen))
    ERR(iRetVal);
  pzAcqStn = (char *)malloc(iStrLen + 1);
  if (iRetVal = nc_get_att_text(iNcid, NC_GLOBAL, "acq_station_name", pzAcqStn))
    ERR(iRetVal);
  pzAcqStn[iStrLen] = '\0';
  if (OUT) (void)printf("Acquisition station name: %s\n", pzAcqStn);

/* Get cycle number from global attributes */
  if (iRetVal = nc_get_att_int(iNcid, NC_GLOBAL, "cycle_number", &iCycleNo))
    ERR(iRetVal);
  if (OUT) (void)printf("Cycle number: %d\n", iCycleNo);

/* Get absolute revolution number from global attributes */
  if (iRetVal = nc_get_att_int(iNcid, NC_GLOBAL, "absolute_rev_number", &iAbsRevNo))
    ERR(iRetVal);
  if (OUT) (void)printf("Absolute revolution number: %d\n", iAbsRevNo);

/* Close the file, freeing all resources. */
  if (iRetVal = nc_close(iNcid))
    ERR(iRetVal);
/**********************netCDF code****************/

/* Set up Aquisition Station for 3rd decriptor */
  (void)strncpy(azCVALS[0], pzAcqStn, 80);
  if (strlen(pzAcqStn) >= 80)
  {
    if (OUT) (void)printf("Aquisition station name too long.\n");
    /* Write syslog message for monitoring */
    if (LOG) syslog(LOG_ERR|LOG_LOCAL2, "LOG_ERR: Aquisition station name too long");
    return -102;
  }

/* Set up Software ID for 4th descriptor, and */
/* Numerical Model Identifier for 7th descriptor */
  azCVALS[1][0] =  'U'; azCVALS[1][1] =  'N'; azCVALS[1][2] =  'S'; azCVALS[1][3] =  'P';
  azCVALS[1][4] =  'E'; azCVALS[1][5] =  'C'; azCVALS[1][6] =  'I'; azCVALS[1][7] =  'F';
  azCVALS[1][8] =  'I'; azCVALS[1][9] =  'E'; azCVALS[1][10] = 'D'; azCVALS[1][11] = ' ';
  azCVALS[1][12] = ' '; azCVALS[1][13] = ' '; azCVALS[1][14] = ' '; azCVALS[1][15] = ' ';
  azCVALS[1][16] = ' '; azCVALS[1][17] = ' '; azCVALS[1][18] = ' '; azCVALS[1][19] = ' ';

/* Build output file name */
/* file names defined by WMO Manual on the GTS (WMO No. 386), July 2007, pp A.II-15/31 - 36 */
/* pflag_productidentifier_oflag_originator_yyyyMMddhhmmss[_freeformat].type[.compression] */
  pzOutputFile = calloc(MAX_OUTPUT_FILENAME_LENGTH + 1, 1);
/* Set pflag */
  (void)strcat(pzOutputFile, "W");
/* Insert separator */
  (void)strcat(pzOutputFile, "_");
/* Set location indicator (within product identifier) */
/* ECP 5 - Global attribute processing_center has new values in v2.3 of prod specn */
/*         (MOD/SSE, 25/11/08) */
/* AR 130 - 2 letter coutry code in file name should be in capital letters for ISO 3166 */
/*         (MOD/SSE, 25/06/09) */
/* AR 130 - 2 letter coutry code reset to lower case*/
/*         (MOD/SSE, 07/07/09) */
/* AR 130 - 2 letter coutry code in file name should be in capital letters for ISO 3166 */
/*         (MOD/SSE, 18/01/10) */
  if (strstr(pzProcCen, "EUM") != NULL) /* EUMETSAT */
    (void)strcat(pzOutputFile, "XX-EUMETSAT-Darmstadt");
  else if (strstr(pzProcCen, "NOAA") != NULL) /* NOAA */
    (void)strcat(pzOutputFile, "US-NOAA-Washington");
  else if (strstr(pzProcCen, "ESPC") != NULL) /* NOAA */
    (void)strcat(pzOutputFile, "US-NOAA-Washington");
  else if (strstr(pzProcCen, "CNES") != NULL) /* CNES */
    (void)strcat(pzOutputFile, "FR-CNES-Toulouse");
  else if (strstr(pzProcCen, "SALP") != NULL) /* CNES */
    (void)strcat(pzOutputFile, "FR-CNES-Toulouse");
  else
    (void)strcat(pzOutputFile, "NK-NK-Unknown");
/* Insert separator (within product identifier) */
  (void)strcat(pzOutputFile, ",");
/* Set data designator (within product identifier) */
  (void)strcat(pzOutputFile, "SURFACE+SATELLITE");
/* Insert separator (within product identifier) */
  (void)strcat(pzOutputFile, ",");
/* Set free description (within product identifier) */
/* AR 160 - Zandle IGDR data for support of BoM (MOD/SSE, 01/02/10) */
  if (strstr(azJ2FileName, "JA2_OPN") != NULL) /* OGDR */
    (void)strcat(pzOutputFile, "JASON2+OGDR");
  else if (strstr(azJ2FileName, "JA2_IPN") != NULL) /* IGDR */
    (void)strcat(pzOutputFile, "JASON2+IGDR");
  else
    (void)strcat(pzOutputFile, "JASON2+XGDR");
/* Insert separator */
  (void)strcat(pzOutputFile, "_");
/* Set oflag */
  (void)strcat(pzOutputFile, "C");
/* Insert separator */
  (void)strcat(pzOutputFile, "_");
/* Set originator */
/* ECP 5 - Global attribute processing_center has new values in v2.3 of prod specn */
/*         (MOD/SSE, 25/11/08) */
  if (strstr(pzProcCen, "EUM") != NULL) /* EUMETSAT */
    (void)strcat(pzOutputFile, "EUMS");
  else if (strstr(pzProcCen, "NOAA") != NULL) /* NOAA */
    (void)strcat(pzOutputFile, "KNES");
  else if (strstr(pzProcCen, "ESPC") != NULL) /* NOAA */
    (void)strcat(pzOutputFile, "KNES");
  else if (strstr(pzProcCen, "CNES") != NULL) /* CNES */
    (void)strcat(pzOutputFile, "LFPW");
  else if (strstr(pzProcCen, "SALP") != NULL) /* CNES */
    (void)strcat(pzOutputFile, "LFPW");
  else
    (void)strcat(pzOutputFile, "CCCC");
/* Insert separator */
  (void)strcat(pzOutputFile, "_");
/* Set creation date */
  (void)strncat(pzOutputFile, pzCreatTime, 4);
  (void)strncat(pzOutputFile, pzCreatTime + 5, 2);
  (void)strncat(pzOutputFile, pzCreatTime + 8, 2);
/* Set creation time */
  (void)strncat(pzOutputFile, pzCreatTime + 11, 2);
  (void)strncat(pzOutputFile, pzCreatTime + 14, 2);
  (void)strncat(pzOutputFile, pzCreatTime + 17, 2);
/* Insert separator */
  (void)strcat(pzOutputFile, "_");
/* Set product version */
  (void)strncat(pzOutputFile, azProdVers, 1);
/* Insert separator */
  (void)strcat(pzOutputFile, "_");
/* Set cycle number */
  (void)strncat(pzOutputFile, azCycle, 3);
/* Insert separator */
  (void)strcat(pzOutputFile, "_");
/* Set pass number */
  (void)strncat(pzOutputFile, azPass, 3);
/* Insert separator */
  (void)strcat(pzOutputFile, "_");
/* Set stop time */
  (void)strncat(pzOutputFile, azStopTime, 8);
  (void)strncat(pzOutputFile, &(azStopTime[9]), 6);
/* Insert separator */
  (void)strcat(pzOutputFile, ".");
/* Set type */
  (void)strcat(pzOutputFile, "bin");

/* Dump WMO file name if requested by command line argument */
  if (1 == siComLinStatus)      /* wmo-file-name */
  {
    (void)printf("WMO file name: ");
    (void)printf("%s\n", pzOutputFile);
    return 0;
  }

/* Write syslog message for monitoring */
  if (LOG) syslog(LOG_INFO|LOG_LOCAL2, "LOG_INFO: WMO file name: %s",
     pzOutputFile);

/* Open output file */
  if ((fpBUFROut = fopen(pzOutputFile, "w+")) == NULL)
  {
    if (OUT) (void)printf("Cant open ouput file\n");
    /* Write syslog message for monitoring */
    if (LOG) syslog(LOG_ERR|LOG_LOCAL2, "LOG_ERR: Cant open ouput file",
       siStatus);
    return -103;
  }

/* Initialise point count in bulletin */
  siPointInBulletin = 0;

/* Initialise bulletin number sequence */
  siBulletinSeq = 1;

/* State need for call to prepare for call to BUFR encoder */
  ucPrepReq = 1;

/* Set dummy spacecraft */
  usiSpaceCraftID = 4;

/* Loop around points */
  for (usiFOV = 0; usiFOV < iNcLength; usiFOV++)
  {

/* Prepare for call to BUFR encoder if required */
    if (1 == ucPrepReq)
    {
      vBUFREncPrep(pzProcCen, pzCreatTime,
        aliKSEC0, aliKSEC1, aliKSEC2, aliKSEC3, aliKSEC4,
        &liKTDLEN, aliKTDLST, &liKDLEN, &liKELEM, &liKVALS, adVALUES);
      ucPrepReq = 0;
    }

/* Pack JASON2 OGDR data ready for encoding */
/* AR 63 - send radiometer state flag, which is available in real JASON2 data */
/*         (MOD/SSE, 11/06/08) */
/* AR 160 - Pack additional parameters (MOD/SSE, 18/01/10) */
    vPackLineJ2OGDR(&siPointInBulletin, &usiFOV, &iCycleNo, &iAbsRevNo,
      psiTb187Vals, psiTb238Vals, psiTb340Vals, pdTimeVals,
      piLatVals, piLonVals, pscSurfVals, pscAltEchoVals, pscRadSurfVals,
      psiWSpdUVals, psiWSpdVVals, psiWSpdAltVals, psiWSpdRadVals,
      psiRadWVVals, psiRadLWVals, piMeanDynTopVals, piHeightVals,
      psiInstAltRVals, psiOffNadPtfVals, psiOffNadWvVals, psiModIonCorVals,
      psiAltIonCorVals, psiMDryTCorVals, psiMWetTCorVals, psiRadWetTCorVals,
      pscIntMSSVals, pscTSol1FVals, pscTSol2FVals, pscInterMetFVals,
      pscOrbStatFVals, pscRadStatFVals, pscAltStatFVals, pscAltKuRFVals, pscAltCRFVals,
      pscAltKuSWHFVals, pscAltCSWHFVals, pscAltKuSigFVals, pscAltCSigFVals,
      pscOffNadKuFVals, pscOffNafPtfFVals, pscCorKuRFVals, pscCorCRFVals,
      pscCorKuSWHFVals, pscCorCSWHFVals, pscCorKuSigFVals, pscCorCSigFVals,
      pscRainFVals, pscQTb187FVals, pscQTb238FVals, pscQTb340FVals,
      pscTbInterFVals, pscIceVals, piRangeKuVals, psiRMSRanKuVals,
      pscNumRanKuVals, piInstCorrKuVals, psiSSBCorrKuVals, psiSWHKuVals,
      psiRMSSWHKuVals, pscNumSWHKuVals, psiInstCorrSWHKuVals, psiSig0KuVals,
      psiRMSSig0KuVals, pscNumSig0KuVals, psiInstCorrSig0KuVals,
      pscAtCorrSig0KuVals, psiAGCKuVals, psiRMSAGCKuVals, pscNumAGCKuVals,
      piRangeCVals, psiRMSRanCVals, pscNumRanCVals, piInstCorrCVals,
      psiSSBCorrCVals, psiSWHCVals, psiRMSSWHCVals, pscNumSWHCVals,
      psiInstCorrSWHCVals, psiSig0CVals, psiRMSSig0CVals, pscNumSig0CVals,
      psiInstCorrSig0CVals, pscAtCorrSig0CVals, psiAGCCVals, psiRMSAGCCVals,
      pscNumAGCCVals, piMSSVals, piMSSAltVals, piGeoidVals, piBathyVals,
      psiSolidTideVals, piGeoTide1Vals, piGeoTide2Vals, psiLoadTide1Vals,
      psiLoadTide2Vals, psiOcTideEqVals, psiOcTideNonEqVals, psiPoleTideVals,
      psiInvBathVals, psiHfFlucVals, pscASFCBandVals, pscASFCBandStatVals,
      pscASFKuBandStatVals, pscASFBandSeqVals, pscECMWFMapAvlVals,
      pscInterpFlagMDTVals, psiSSHAVals, &liKELEM, adVALUES);

/* Increment line count */
    siPointInBulletin++;

/* If limit of FOVs per bulletin */
    if ((FOV_PER_BULLETIN - 1) == siPointInBulletin)
    {

/* Pack data */
      aliKSEC3[2] = siPointInBulletin;
      liKVALS = liKELEM * siPointInBulletin;
/* AR 52: Set identifying time in Scetion 1 to correspond to earliest data */
/*        time (MOD/SSE, 28/04/08) */
      aliKSEC1[8] = (long int)adVALUES[7];    /* Year */
      aliKSEC1[9] = (long int)adVALUES[8];    /* Month */
      aliKSEC1[10] = (long int)adVALUES[9];   /* Date */
      aliKSEC1[11] = (long int)adVALUES[10];  /* Hour */
      aliKSEC1[12] = (long int)adVALUES[11];  /* Minute */
      aliKSEC1[17] = (long int)adVALUES[12];  /* Second */
      (void)bufren(
        aliKSEC0,  aliKSEC1,  aliKSEC2,  aliKSEC3,
        aliKSEC4,  &liKTDLEN, aliKTDLST, &liKDLEN,
        aliKDATA,  &liKELEM,  &liKVALS,  adVALUES,
        azCVALS,   &liKBUFL,  aliKBUFF,  &liKERR);

/* Check successful completion of BUFREN indicated by liKERR = 0 */
      if (0 != liKERR)
      {
        if (OUT) (void)printf("BUFREN failed with error code KERR = %ld.\n", liKERR);
        if (LOG) syslog(LOG_ERR|LOG_LOCAL2, "LOG_ERR: BUFREN failed with error code KERR = %ld",
          liKERR);
        return liKERR;
      }

/* Calculate length of bulletin from octets 5 - 7, checking endian state */
      if (4 == (aliKBUFF[1] & 0X000000FFL))
        uliTotBUFRLength = (aliKBUFF[1] >> 8) & 0X00FFFFFFL; 
      else
        uliTotBUFRLength = (htonl(aliKBUFF[1]) >> 8) & 0X00FFFFFFL; 

/* Append bulletin to RMDCN type file */
/* Use satellite ID to switch T2 in bulletin header */
      usiDay = (unsigned short int)adVALUES[9];
      usiHour = (unsigned short int)adVALUES[10];
      usiMinute = (unsigned short int)adVALUES[11];
      vWriteBulletin(aliKBUFF, &uliTotBUFRLength, pzProcCen, &siBulletinSeq, 
        &usiDay, &usiHour, &usiMinute, &usiSpaceCraftID, fpBUFROut);

/* Prepare for call to BUFR encoder if required */
      vBUFREncPrep(pzProcCen, pzCreatTime,
        aliKSEC0, aliKSEC1, aliKSEC2, aliKSEC3, aliKSEC4,
        &liKTDLEN, aliKTDLST, &liKDLEN, &liKELEM, &liKVALS, adVALUES);

/* Reset line count and repack last line as first of new bulletin */
      siPointInBulletin = 0;
    }

  }

/* Call BUFR encoder to pack remaining lines */
  aliKSEC3[2] = siPointInBulletin;
  liKVALS = liKELEM * siPointInBulletin;
/* AR 52: Set identifying time in Scetion 1 to correspond to earliest data */
/*        time (MOD/SSE, 28/04/08) */
  aliKSEC1[8] = (long int)adVALUES[7];    /* Year */
  aliKSEC1[9] = (long int)adVALUES[8];    /* Month */
  aliKSEC1[10] = (long int)adVALUES[9];   /* Date */
  aliKSEC1[11] = (long int)adVALUES[10];  /* Hour */
  aliKSEC1[12] = (long int)adVALUES[11];  /* Minute */
  aliKSEC1[17] = (long int)adVALUES[12];  /* Second */
  (void)bufren(
    aliKSEC0,  aliKSEC1,  aliKSEC2,  aliKSEC3,
    aliKSEC4,  &liKTDLEN, aliKTDLST, &liKDLEN,
    aliKDATA,  &liKELEM,  &liKVALS,  adVALUES,
    azCVALS,   &liKBUFL,  aliKBUFF,  &liKERR);

/* Check successful completion of BUFREN indicated by liKERR = 0 */
  if (0 != liKERR)
  {
    if (OUT) (void)printf("BUFREN failed with error code KERR = %ld.\n", liKERR);
    if (LOG) syslog(LOG_ERR|LOG_LOCAL2, "LOG_ERR: BUFREN failed with error code KERR = %ld",
      liKERR);
    return liKERR;
  }

/* Calculate length of bulletin from octets 5 - 7, checking endian state */
  if (4 == (aliKBUFF[1] & 0X000000FFL))
    uliTotBUFRLength = (aliKBUFF[1] >> 8) & 0X00FFFFFFL; 
  else
    uliTotBUFRLength = (htonl(aliKBUFF[1]) >> 8) & 0X00FFFFFFL; 


/* Append bulletin to RMDCN type file */
/* Use satellite ID to switch T2 in bulletin header */
  usiDay = (unsigned short int)adVALUES[9];
  usiHour = (unsigned short int)adVALUES[10];
  usiMinute = (unsigned short int)adVALUES[11];
  vWriteBulletin(aliKBUFF, &uliTotBUFRLength, pzProcCen, &siBulletinSeq, 
    &usiDay, &usiHour, &usiMinute, &usiSpaceCraftID, fpBUFROut);

/* Finished! */
  if (OUT) (void)printf("Finished.\n");
/* Write syslog message for monitoring */
  if (LOG) syslog(LOG_INFO|LOG_LOCAL2, "LOG_INFO: NetCDF to BUFR translation completed nominally");

/* Append dummy message (only 10 character length) to output file */
  (void)fprintf(fpBUFROut,"0000000000");

/* Close output file */
  fclose(fpBUFROut);

/* Create hard link to output file with name like the incoming file */
/* but with 7th character being "B" rather than "N".  As requested by */
/* NOAA in January 2008 */
  if((pzSubString = strstr(azJ2FileName,"JA2_OPN")) != NULL)
  {
    iStrIndex = azJ2FileName - pzSubString + 6;
    azJ2FileName[iStrIndex] = 'B';
    link(pzOutputFile, azJ2FileName);
  }
/* AR 160 - Also handle IGDR data for support of BoM (MOD/SSE, 01/02/10) */
  else if((pzSubString = strstr(azJ2FileName,"JA2_IPN")) != NULL)
  {
    iStrIndex = azJ2FileName - pzSubString + 6;
    azJ2FileName[iStrIndex] = 'B';
    link(pzOutputFile, azJ2FileName);
  }

/* Free memory used for file name */
  (void)free(pzOutputFile);  

/* Free up memory for time */
  (void)free(pdTimeVals);

/* Free up memory for lat, lon and surface */
  (void)free(piLatVals);
  (void)free(piLonVals);
  (void)free(pscSurfVals);
  (void)free(pscAltEchoVals);
  (void)free(pscRadSurfVals);

/* Free up memory for flags */
  (void)free(pscIntMSSVals);
  (void)free(pscTSol1FVals);
  (void)free(pscTSol2FVals);
  (void)free(pscInterMetFVals);
/* AR 63 - free radiometer state flag, which is available in real JASON2 data */
/*         (MOD/SSE, 11/06/08) */
  (void)free(pscOrbStatFVals);
  (void)free(pscRadStatFVals);
  (void)free(pscAltStatFVals);
  (void)free(pscAltKuRFVals);
  (void)free(pscAltCRFVals);
  (void)free(pscAltKuSWHFVals);
  (void)free(pscAltCSWHFVals);
  (void)free(pscAltKuSigFVals);
  (void)free(pscAltCSigFVals);
  (void)free(pscOffNadKuFVals);
  (void)free(pscOffNafPtfFVals);
  (void)free(pscCorKuRFVals);
  (void)free(pscCorCRFVals);
  (void)free(pscCorKuSWHFVals);
  (void)free(pscCorCSWHFVals);
  (void)free(pscCorKuSigFVals);
  (void)free(pscCorCSigFVals);
  (void)free(pscRainFVals);
  (void)free(pscQTb187FVals);
  (void)free(pscQTb238FVals);
  (void)free(pscQTb340FVals);
  (void)free(pscTbInterFVals);
  (void)free(pscIceVals);

/* Free up memory for Ku band altimeter data */
  (void)free(piRangeKuVals);
  (void)free(psiRMSRanKuVals);
  (void)free(pscNumRanKuVals);
  (void)free(piInstCorrKuVals);
  (void)free(psiSSBCorrKuVals);
  (void)free(psiSWHKuVals);
  (void)free(psiRMSSWHKuVals);
  (void)free(pscNumSWHKuVals);
  (void)free(psiInstCorrSWHKuVals);
  (void)free(psiSig0KuVals);
  (void)free(psiRMSSig0KuVals);
  (void)free(pscNumSig0KuVals);
  (void)free(psiInstCorrSig0KuVals);
  (void)free(pscAtCorrSig0KuVals);
  (void)free(psiAGCKuVals);
  (void)free(psiRMSAGCKuVals);
  (void)free(pscNumAGCKuVals);

/* Free up memory for C band altimeter data */
  (void)free(piRangeCVals);
  (void)free(psiRMSRanCVals);
  (void)free(pscNumRanCVals);
  (void)free(piInstCorrCVals);
  (void)free(psiSSBCorrCVals);
  (void)free(psiSWHCVals);
  (void)free(psiRMSSWHCVals);
  (void)free(pscNumSWHCVals);
  (void)free(psiInstCorrSWHCVals);
  (void)free(psiSig0CVals);
  (void)free(psiRMSSig0CVals);
  (void)free(pscNumSig0CVals);
  (void)free(psiInstCorrSig0CVals);
  (void)free(pscAtCorrSig0CVals);
  (void)free(psiAGCCVals);
  (void)free(psiRMSAGCCVals);
  (void)free(pscNumAGCCVals);

/* Free up memory for brightness temperatures */
  (void)free(psiTb187Vals);
  (void)free(psiTb238Vals);
  (void)free(psiTb340Vals);

/* Free up memory for environmental params */
  (void)free(psiWSpdUVals);
  (void)free(psiWSpdVVals); 
  (void)free(psiWSpdAltVals);
  (void)free(psiWSpdRadVals); 
  (void)free(psiRadWVVals);
  (void)free(psiRadLWVals); 

/* Free up memory for topography */
  (void)free(piMeanDynTopVals);
  (void)free(piHeightVals);
  (void)free(psiInstAltRVals);
  (void)free(psiOffNadPtfVals);
  (void)free(psiOffNadWvVals);
  (void)free(psiModIonCorVals);
  (void)free(psiAltIonCorVals);
  (void)free(psiMDryTCorVals);
  (void)free(psiMWetTCorVals);
  (void)free(psiRadWetTCorVals);

/* Free up memory for geophysical parameters */
  (void)free(piMSSVals);
/* AR 63 - variable mss_tp_along_track not available in J2 O/IGDRs (MOD/SSE, 11/06/08) */
/*
  (void)free(piMSSAltVals);
*/
  (void)free(piGeoidVals);
  (void)free(piBathyVals);
  (void)free(psiSolidTideVals);
  (void)free(piGeoTide1Vals);
  (void)free(piGeoTide2Vals);
  (void)free(psiLoadTide1Vals);
  (void)free(psiLoadTide2Vals);
  (void)free(psiOcTideEqVals);
  (void)free(psiOcTideNonEqVals);
  (void)free(psiPoleTideVals);
  (void)free(psiInvBathVals);
  (void)free(psiHfFlucVals);

/* AR 160 - Free memory for additional parameters parameters (MOD/SSE, 18/01/10) */
  (void)free(pscASFCBandVals);
  (void)free(pscASFCBandStatVals);
  (void)free(pscASFKuBandStatVals);
  (void)free(pscASFBandSeqVals);
  (void)free(pscECMWFMapAvlVals);
  (void)free(pscInterpFlagMDTVals);
  (void)free(psiSSHAVals);

/* Free up memory for global attributes */
  (void)free(pzProcCen);
  (void)free(pzCreatTime);
  (void)free(pzAcqStn);
}

short int siCommandLineOptions(int *argc, char *argv[])
{

/* Check number of arguments */
  if (*argc < 2)
  {
    if (OUT)(void)printf("Wrong number of arguments: %d, there should be 1 or 2.\n", (*argc)--);
    return -201;
  }
  else if (*argc > 3)
  {
    if (OUT)(void)printf("Wrong number of arguments: %d, there should be 1 or 2.\n", (*argc)--);
    return -201;
  }

/* IF argument starts with "--" THEN*/
  if (0 == (*(argv + 1) - strstr(*(argv + 1), "--")))
  {
/*   IF argument is "--wmo-file-name" THEN */
    if (strstr(*(argv + 1), "--wmo-file-name") != NULL)
    {
/*     Return function value of 1 */
      if (OUT)(void)printf("Command line request for wmo-file-name\n");
      (*argc)--;
      return 1;
    }
/*   ELSE IF is "--fist-version" THEN */
    else if (strstr(*(argv + 1), "--fist-version") != NULL)
    {
/*     Return function value of 2 */
      if (OUT)(void)printf("Command line request for fist-version\n");
      (*argc)--;
      return 2;
    }
/*   ELSE IF is "--bufr-table-versions" THEN */
    else if (strstr(*(argv + 1), "--bufr-table-versions") != NULL)
    {
/*     Return function value of 3 */
      if (OUT)(void)printf("Command line request for bufr-table-versions\n");
      (*argc)--;
      return 3;
    }
/*   ELSE IF argument is "--ecmwf-library-version" THEN */
    else if (strstr(*(argv + 1), "--ecmwf-library-version") != NULL)
    {
/*     Return function value of 4 */
      if (OUT)(void)printf("Command line request for ecmwf-library-version\n");
      (*argc)--;
      return 4;
    }
/*   ELSE */
    else
    {
/*     Unrecognised command line argument */
      if (OUT)(void)printf("Command line option [%s] not supported\n", *(argv + 1));
      (*argc)--;
      return 5;
    }
  }
/* ELSE */
  else
  {
/*   Return nominal function value of 0 */
    return 0;
  }
}


short int siParseInputFileName(int argc, char *argv[], char azJ2FileName[],
  char azProdVers[], char azCycle[], short int *siCycle,
  char azPass[], short int *siPass, char azStopTime[])
{

/* File names are described in TP3-JS-IF-200-CNES, Issue 2, Revision 1, 5/10/7 */
/* File name is of form JA2_OPN_2PvPccc_ppp_YYYYMMDD_HHMMSS_yyyymmdd_hhmmss */
/* with v: product version */
/* with ccc: cycle number at start time */
/* with ppp: pass number at start time */
/* with yyyymmdd_hhmmss: stop time (last measurement date) */

/* Check number of arguments */
  if (argc < 2)
  {
    if (OUT)(void)printf("Wrong number of remaining arguments: %d, there should be 1.\n", --argc);
    return -301;
  }
  else if (argc > 2)
  {
    if (OUT)(void)printf("Wrong number of remaining arguments: %d, there should be 1.\n", --argc);
    if (OUT)(void)printf("Assuming input file name is first argument\n");
  }

/* Parse input file name */
  (void)strncpy(azJ2FileName, *(argv + 1), BUFSIZ);
  if (strlen(azJ2FileName) >= BUFSIZ)
  {
    if (OUT)(void)printf("Input file name too long.\n");
    return -302;
  }
  if (OUT)(void)printf("azJ2FileName is %s.\n", azJ2FileName);

/* File names are described in TP3-JS-IF-200-CNES, Issue 2, Revision 1, 5/10/7 */
/* File name is of form JA2_OPN_2PvPccc_ppp_YYYYMMDD_HHMMSS_yyyymmdd_hhmmss */
/* with v: product version (letter) */
/* with ccc: cycle number at start time */
/* with ppp: pass number at start time */
/* with yyyymmdd_hhmmss: stop time (last measurement date) */

/* AR 160 - Also allow processing of IGDR data for BoM (MOD/SSE, 01/02/10) */
/* Perform basic validity check on file name */
  if ((NULL == strstr(azJ2FileName, "JA2_OPN_2P")) &&
      (NULL == strstr(azJ2FileName, "JA2_IPN_2P")))
  {
    if (OUT)(void)printf("File name does not contain JA2_OPN_2P or JA2_OPN_2P\n");
    return -303;
  }
  else if (strlen(azJ2FileName) !=
    strlen("JA2_OPN_2PvPccc_ppp_YYYYMMDD_HHMMSS_yyyymmdd_hhmmss"))
  {
    if (OUT)(void)printf("File name length is wrong: %d, should be %d.\n",
      strlen(azJ2FileName), 
      strlen("JA2_OPN_2PvPccc_ppp_YYYYMMDD_HHMMSS_yyyymmdd_hhmmss"));
    return -304;
  }
  else
  {
    if (OUT)(void)printf("File name seems valid\n");
  }

/* Extract product version from file name */
  (void *)strncpy(azProdVers, &(azJ2FileName[10]), 1);
  azProdVers[1] = '\0';

/* Extract cycle number at start time from file name */
  (void *)strncpy(azCycle, &(azJ2FileName[12]), 3); 
  azCycle[3] = '\0';
  *siCycle = (azJ2FileName[12] - '0') * 100;
  *siCycle += (azJ2FileName[13] - '0') * 10;
  *siCycle += (azJ2FileName[14] - '0');

/* Extract pass number at start time from file name */
  (void *)strncpy(azPass, &(azJ2FileName[16]), 3);
  azPass[3] = '\0';
  *siPass = (azJ2FileName[16] - '0') * 100;
  *siPass += (azJ2FileName[17] - '0') * 10;
  *siPass += (azJ2FileName[18] - '0');

/* Extract stop time from file name */
  (void *)strncpy(azStopTime, &(azJ2FileName[36]), 15);
  azStopTime[15] = '\0';

/* Successful completion */
  if (OUT)(void)printf("Product version: %1s, Cycle %3.3d, Pass %3.3d, Stop time: %15s\n",
    azProdVers, *siCycle, *siPass, azStopTime);
  return 0;
}

void vBUFREncPrep(char *pzProcCen, char *pzCreatTime,
  int32_t aliKSEC0[], int32_t aliKSEC1[],
  int32_t aliKSEC2[], int32_t aliKSEC3[], int32_t aliKSEC4[],
  int32_t *liKTDLEN, int32_t aliKTDLST[], int32_t *liKDLEN,
  int32_t *liKELEM, int32_t *liKVALS, double adVALUES[])
{
  short int siStatus = 0;
  long int  liLoop;
  unsigned short int usiYear, usiMonth, usiDay, usiHour, usiMinute, usiSec;

  usiYear = (pzCreatTime[7] - 48) * 1000;
  usiYear += (pzCreatTime[8] - 48) * 100;
  usiYear += (pzCreatTime[9] - 48) * 10;
  usiYear += (pzCreatTime[10] - 48);
  if (strstr(pzCreatTime, "JAN") != NULL) usiMonth = 1;
  else if (strstr(pzCreatTime, "FEB") != NULL) usiMonth = 2;
  else if (strstr(pzCreatTime, "MAR") != NULL) usiMonth = 3;
  else if (strstr(pzCreatTime, "APR") != NULL) usiMonth = 4;
  else if (strstr(pzCreatTime, "MAY") != NULL) usiMonth = 5;
  else if (strstr(pzCreatTime, "JUN") != NULL) usiMonth = 6;
  else if (strstr(pzCreatTime, "JUL") != NULL) usiMonth = 7;
  else if (strstr(pzCreatTime, "AUG") != NULL) usiMonth = 8;
  else if (strstr(pzCreatTime, "SEP") != NULL) usiMonth = 9;
  else if (strstr(pzCreatTime, "OCT") != NULL) usiMonth = 10;
  else if (strstr(pzCreatTime, "NOV") != NULL) usiMonth = 11;
  else usiMonth = 12;
  usiDay = (pzCreatTime[0] - 48) * 10;
  usiDay += (pzCreatTime[1] - 48);
  usiHour = (pzCreatTime[12] - 48) * 10;
  usiHour += (pzCreatTime[13] - 48);
  usiMinute = (pzCreatTime[15] - 48) * 10;
  usiMinute += (pzCreatTime[16] - 48);
  usiSec = (pzCreatTime[18] - 48) * 10;
  usiSec += (pzCreatTime[19] - 48);

/* Set Section 0 */
  aliKSEC0[0] = 0;            /* length of section 0 in bytes (set to zero) */
  aliKSEC0[1] = 0;            /* total length of message */
  aliKSEC0[2] = BUFR_EDITION; /* BUFR edition number */

/* Set Section 1 */
  aliKSEC1[0] = 22;           /* length of section 1 in bytes */
  aliKSEC1[1] = BUFR_EDITION; /* BUFR edition number */
/* ECP 5 - Global attribute processing_center has new values in v2.3 of prod specn */
/*         (MOD/SSE, 25/11/08) */
  if (strstr(pzProcCen, "EUM") != NULL) /* EUMETSAT */
    aliKSEC1[2] = ORIG_CENTRE_EUM;  /* Originating Centre: EUMETSAT */
  else if (strstr(pzProcCen, "NOAA") != NULL) /* NOAA */
    aliKSEC1[2] = ORIG_CENTRE_NOAA;  /* Originating Centre: NOAA */
  else if (strstr(pzProcCen, "ESPC") != NULL) /* NOAA */
    aliKSEC1[2] = ORIG_CENTRE_NOAA;  /* Originating Centre: NOAA */
  else if (strstr(pzProcCen, "CNES") != NULL) /* CNES */
    aliKSEC1[2] = ORIG_CENTRE_CNES;  /* Originating Centre: CNES */
  else if (strstr(pzProcCen, "SALP") != NULL) /* CNES */
    aliKSEC1[2] = ORIG_CENTRE_CNES;  /* Originating Centre: CNES */
  else
    aliKSEC1[2] = ORIG_CENTRE_EUM;  /* Default originating Centre: EUMETSAT */
  aliKSEC1[3] = 0;            /* Update sequence number */
  aliKSEC1[4] = 0;            /* Flag to show no Section 2 */
  aliKSEC1[5] = 12;           /* Data category: Surface data (satellite) */
  aliKSEC1[6] = 99;           /* Arbitrary local sub category */
  aliKSEC1[7] = BUFR_LT_VERSION;            /* Local Table version: none used */
  aliKSEC1[8] = usiYear;     /* Year */
  aliKSEC1[9] = usiMonth;    /* Month */
  aliKSEC1[10] = usiDay;     /* Date */
  aliKSEC1[11] = usiHour;    /* Hour */
  aliKSEC1[12] = usiMinute;  /* Minute */
  aliKSEC1[13] = 0;           /* Master Table number: Meteorology */
  aliKSEC1[14] = BUFR_MT_VERSION;           /* Master Table version */
  aliKSEC1[15] = 0;           /* Originating SubCentre: None */
  aliKSEC1[16] = 255;         /* INTERNATIONAL SUB-CATEGORY (CCT C13) */
  aliKSEC1[17] = usiSec;     /* Second */

/* Set Section 2 */
  for (liLoop = 0; liLoop < 64; liLoop++)
    aliKSEC2[liLoop] = 0;

/* Set Section 3 */
  aliKSEC3[0] = 0;    /* Length of section 3 in bytes (set to 0) */
  aliKSEC3[1] = 0;    /* Reserved */
  aliKSEC3[2] = 0;    /* Set in main program before calling BUFREN */
  aliKSEC3[3] = 192;  /* BIT 8 (=128) is data type flag */
                      /* BIT 7 (=64) is comrepession flag */

/* Set Section 4 */
  aliKSEC4[0] = 0;    /* Length of section 4 in bytes (set to 0) */
  aliKSEC4[1] = 0;    /* Reserved */

/* liKVALS is set in main program before calling BUFREN */
  *liKVALS = 0; /* Set in main program before calling BUFREN */

/* Define remaining input parameters */
/* AR 160 - Use new template with extra descriptors (MOD/SSE, 18/01/10) */
  *liKTDLEN = 1;/* Number of descriptors for section 3 */
  aliKTDLST[0] =  340010;

  *liKELEM = 104; /* Number of expanded data descriptors per subset */

  *liKDLEN = 0; /* Size of delayed replication data array (none) */

/* Blank out unset values */
  for (liLoop = 1; liLoop < 180000; liLoop++)
    aliKTDLST[liLoop] = 0;
  for (liLoop = 0; liLoop < 800000; liLoop++)
    aliKBUFF[liLoop] = 0;
  for (liLoop = 0; liLoop < 2500000; liLoop++)
    adVALUES[liLoop] = BUFR_MISSING;
}

/* Pack JASON2 OGDR data ready for encoding */
/* AR 63 - use radiometer state flag, which is available in real JASON2 data */
/*         (MOD/SSE, 11/06/08) */
/* AR 160 - Pack additional parameters (MOD/SSE, 18/01/10) */
void vPackLineJ2OGDR(short int *siPointInBulletin,
  unsigned short int *usiFOV, int *iCycleNo, int *iAbsRevNo,
  short int *psiTb187Vals, short int *psiTb238Vals, short int *psiTb340Vals,
  double *pdTimeVals, int *piLatVals, int *piLonVals,
  signed char *pscSurfVals, signed char *pscAltEchoVals, signed char *pscRadSurfVals,
  short int *psiWSpdUVals, short int *psiWSpdVVals, short int *psiWSpdAltVals,
  short int *psiWSpdRadVals, short int *psiRadWVVals, short int *psiRadLWVals,
  int *piMeanDynTopVals, int *piHeightVals, short int *psiInstAltRVals,
  short int *psiOffNadPtfVals, short int *psiOffNadWvVals, short int *psiModIonCorVals,
  short int *psiAltIonCorVals, short int *psiMDryTCorVals, short int *psiMWetTCorVals,
  short int *psiRadWetTCorVals, signed char *pscIntMSSVals,
  signed char *pscTSol1FVals, signed char *pscTSol2FVals, signed char *pscInterMetFVals,
  signed char *pscOrbStatFVals, signed char *pscRadStatFVals,
  signed char *pscAltStatFVals, signed char *pscAltKuRFVals,
  signed char *pscAltCRFVals, signed char *pscAltKuSWHFVals, signed char *pscAltCSWHFVals,
  signed char *pscAltKuSigFVals, signed char *pscAltCSigFVals, signed char *pscOffNadKuFVals,
  signed char *pscOffNafPtfFVals, signed char *pscCorKuRFVals, signed char *pscCorCRFVals,
  signed char *pscCorKuSWHFVals, signed char *pscCorCSWHFVals, signed char *pscCorKuSigFVals,
  signed char *pscCorCSigFVals, signed char *pscRainFVals, signed char *pscQTb187FVals,
  signed char *pscQTb238FVals, signed char *pscQTb340FVals, signed char *pscTbInterFVals,
  signed char *pscIceVals, int *piRangeKuVals, short int *psiRMSRanKuVals,
  signed char *pscNumRanKuVals, int *piInstCorrKuVals, short int *psiSSBCorrKuVals,
  short int *psiSWHKuVals, short int *psiRMSSWHKuVals, signed char *pscNumSWHKuVals,
  short int *psiInstCorrSWHKuVals, short int *psiSig0KuVals, short int *psiRMSSig0KuVals,
  signed char *pscNumSig0KuVals, short int *psiInstCorrSig0KuVals,
  signed char *pscAtCorrSig0KuVals, short int *psiAGCKuVals, short int *psiRMSAGCKuVals,
  signed char *pscNumAGCKuVals, int *piRangeCVals, short int *psiRMSRanCVals,
  signed char *pscNumRanCVals, int *piInstCorrCVals, short int *psiSSBCorrCVals,
  short int *psiSWHCVals, short int *psiRMSSWHCVals, signed char *pscNumSWHCVals,
  short int *psiInstCorrSWHCVals, short int *psiSig0CVals, short int *psiRMSSig0CVals,
  signed char *pscNumSig0CVals, short int *psiInstCorrSig0CVals,
  signed char *pscAtCorrSig0CVals, short int *psiAGCCVals, short int *psiRMSAGCCVals,
  signed char *pscNumAGCCVals, int *piMSSVals, int *piMSSAltVals, int *piGeoidVals,
  int *piBathyVals, short int *psiSolidTideVals, int *piGeoTide1Vals, int *piGeoTide2Vals,
  short int *psiLoadTide1Vals, short int *psiLoadTide2Vals, short int *psiOcTideEqVals,
  short int *psiOcTideNonEqVals, short int *psiPoleTideVals, short int *psiInvBathVals,
  short int *psiHfFlucVals, signed char *pscASFCBandVals, signed char *pscASFCBandStatVals,
  signed char *pscASFKuBandStatVals, signed char *pscASFBandSeqVals,
  signed char *pscECMWFMapAvlVals, signed char *pscInterpFlagMDTVals,
  short int *psiSSHAVals, int32_t *liKELEM, double adVALUES[])
{
  long int   liPointer;
  short int  siParamLoop;
  long int   liFOVSecs;
  double     dFracSecs;
  unsigned short int usiWorkBits;
  unsigned short int usiFOVYear, usiFOVMonth, usiFOVDay;
  unsigned short int usiFOVHour, usiFOVMinute, usiFOVSec;

/* Insert data in BUFR encoding array */
  liPointer = *siPointInBulletin * (*liKELEM);

/* Set satellite information */
  adVALUES[liPointer + 0]  = (double)SAT_ID; /* 261 = JASON2 */
  adVALUES[liPointer + 1]  = (double)INST_ID; /* 366 = JMR */
  adVALUES[liPointer + 2]  = (double)1020.0; /* Station (index 1 in CVALS array, lenght 20) */
  adVALUES[liPointer + 3]  = (double)2012.0; /*  Software (index 2 in CVALS array, lenght 12) */
  adVALUES[liPointer + 4]  = (double)*iCycleNo; /* Sat cycle */
  adVALUES[liPointer + 5]  = (double)*iAbsRevNo; /* Orbit number */
  adVALUES[liPointer + 6]  = (double)2014.0; /* Model ID (index 2 in CVALS array, lenght 14) */

/* Set datation */
  liFOVSecs = (long int)floor(pdTimeVals[*usiFOV]);
  dFracSecs = pdTimeVals[*usiFOV] - floor(pdTimeVals[*usiFOV]);
  vOGDRSecstoYMDHMS(liFOVSecs,
    &usiFOVYear, &usiFOVMonth, &usiFOVDay, &usiFOVHour, &usiFOVMinute, &usiFOVSec);
  dFracSecs += (double)usiFOVSec;
  adVALUES[liPointer + 7]  = (double)usiFOVYear; /* Year */
  adVALUES[liPointer + 8]  = (double)usiFOVMonth; /* Month */
  adVALUES[liPointer + 9]  = (double)usiFOVDay; /* Date */
  adVALUES[liPointer + 10] = (double)usiFOVHour; /* Hour */
  adVALUES[liPointer + 11] = (double)usiFOVMinute; /* Minute */
  adVALUES[liPointer + 12] = dFracSecs; /* Seconds */

/* Set location and surface type */
/* AR 63 - set fill values in netCDF to missing in BUFR (MOD/SSE, 16/06/08) */
  if (FILL_INT_J2 == piLatVals[*usiFOV])
    adVALUES[liPointer + 13] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 13] = (double)piLatVals[*usiFOV] / 1.0E6; /* Latitude */
  if (FILL_INT_J2 == piLonVals[*usiFOV])
    adVALUES[liPointer + 14] = (double)BUFR_MISSING;
  else
/* AR 102 - but BUFR longitudes into range -180 to 180 (MOD/SSE, 13/11/08) */
  {
    adVALUES[liPointer + 14] = (double)piLonVals[*usiFOV] / 1.0E6; /* Longitude */
    if (adVALUES[liPointer + 14] > 180.0) adVALUES[liPointer + 14] -= 360.0;
  }
  if (pscSurfVals[*usiFOV] <= 3)                         /* RS surface type */
    adVALUES[liPointer + 15] = (double)pscSurfVals[*usiFOV];  /* BUFR C/T 0-08-029 */
  else
    adVALUES[liPointer + 15] = (double)BUFR_MISSING;
  if (pscAltEchoVals[*usiFOV] <= 1)                         /* Altimeter echo type */
    adVALUES[liPointer + 16] = (double)pscAltEchoVals[*usiFOV];  /* BUFR C/T 0-08-074 */
  else
    adVALUES[liPointer + 16] = (double)BUFR_MISSING;
  if (0 == pscRadSurfVals[*usiFOV])                          /* sea/ocean */
    adVALUES[liPointer + 17] = (double)1.0;                  /* BUFR C/T 0-08-077 */
  else if (1 == pscRadSurfVals[*usiFOV])                     /* land */
    adVALUES[liPointer + 17] = (double)0.0;                  /* BUFR C/T 0-08-077 */
  else
    adVALUES[liPointer + 17] = (double)BUFR_MISSING;

/* Set flags */
/* AR 63 - set fill values in netCDF to missing in BUFR (MOD/SSE, 16/06/08) */
  usiWorkBits = 0; /* Interpolation flag */
  usiWorkBits |= (pscIntMSSVals[*usiFOV] & 0X0001) << 7;
  usiWorkBits |= (pscTSol1FVals[*usiFOV] & 0X0001) << 6;
  usiWorkBits |= (pscTSol2FVals[*usiFOV] & 0X0001) << 5;
  usiWorkBits |= (pscInterMetFVals[*usiFOV] & 0X0001) << 4;
  adVALUES[liPointer + 18] = (double)usiWorkBits;
  if (FILL_BYTE_J2 == pscOrbStatFVals[*usiFOV]) /* Orbit state flag */
    adVALUES[liPointer + 19] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 19] = (double)(pscOrbStatFVals[*usiFOV] + 1);
  if (FILL_BYTE_J2 == pscAltStatFVals[*usiFOV]) /* Altimeter state flag */
    adVALUES[liPointer + 20] = (double)BUFR_MISSING;
/* AR 103 - account for bit order difference between netCDF and BUFR */
/*         (MOD/SSE, 13/11/08) */
  else
  {
    if (0 == pscAltStatFVals[*usiFOV])
      adVALUES[liPointer + 20] = (double)0X0000; /* netCDF 0 = nominal/A side = 0 in BUFR */
    else
      adVALUES[liPointer + 20] = (double)0X0002; /* netCDF 1 = redundant/B side = 2 in BUFR */
  }
  usiWorkBits = 0; /* Altimeter data quality flag */
  usiWorkBits |= (pscAltKuRFVals[*usiFOV] & 0X0001) << 8;
  usiWorkBits |= (pscAltCRFVals[*usiFOV] & 0X0001) << 7;
  usiWorkBits |= (pscAltKuSWHFVals[*usiFOV] & 0X0001) << 6;
  usiWorkBits |= (pscAltCSWHFVals[*usiFOV] & 0X0001) << 5;
  usiWorkBits |= (pscAltKuSigFVals[*usiFOV] & 0X0001) << 4;
  usiWorkBits |= (pscAltCSigFVals[*usiFOV] & 0X0001) << 3;
  usiWorkBits |= (pscOffNadKuFVals[*usiFOV] & 0X0001) << 2;
  usiWorkBits |= (pscOffNafPtfFVals[*usiFOV] & 0X0001) << 1;
  adVALUES[liPointer + 21] = (double)usiWorkBits;
  usiWorkBits = 0; /* Altimeter correction quality flag */
  usiWorkBits |= (pscCorKuRFVals[*usiFOV] & 0X0001) << 8;
  usiWorkBits |= (pscCorCRFVals[*usiFOV] & 0X0001) << 7;
  usiWorkBits |= (pscCorKuSWHFVals[*usiFOV] & 0X0001) << 6;
  usiWorkBits |= (pscCorCSWHFVals[*usiFOV] & 0X0001) << 5;
  usiWorkBits |= (pscCorKuSigFVals[*usiFOV] & 0X0001) << 4;
  usiWorkBits |= (pscCorCSigFVals[*usiFOV] & 0X0001) << 3;
  adVALUES[liPointer + 22] = (double)usiWorkBits;
  if (FILL_BYTE_J2 == pscRainFVals[*usiFOV]) /* Rain flag */
    adVALUES[liPointer + 23] = (double)BUFR_MISSING;
/* AR 103 - account for bit order difference between netCDF and BUFR */
/*         (MOD/SSE, 13/11/08) */
  else
  {
    if (0 == pscRainFVals[*usiFOV])
      adVALUES[liPointer + 23] = (double)0X0000; /* netCDF 0 = norain  = 0 in BUFR */
    else
      adVALUES[liPointer + 23] = (double)0X0002; /* netCDF 1 = rain = 2 in BUFR */
  }
/* AR 63 - use radiometer state flag, which is available in real JASON2 data */
/*         (MOD/SSE, 11/06/08) */
  if (FILL_BYTE_J2 == pscRadStatFVals[*usiFOV]) /* Radiometer state flag */
    adVALUES[liPointer + 24] = (double)BUFR_MISSING;
/* AR 103 - account for bit order difference between netCDF and BUFR */
/*         (MOD/SSE, 13/11/08) */
  else
  {
    if (0 == pscRadStatFVals[*usiFOV])
      adVALUES[liPointer + 24] = (double)0X0000; /* netCDF 0 = nominal = 0 in BUFR */
    else
      adVALUES[liPointer + 24] = (double)BUFR_MISSING; /* netCDF 1 = redundant = not in BUFR */
  }
  usiWorkBits = 0; /* Radiometer data quality flag */
  usiWorkBits |= (pscQTb187FVals[*usiFOV] & 0X0001) << 7;
  usiWorkBits |= (pscQTb238FVals[*usiFOV] & 0X0001) << 6;
  usiWorkBits |= (pscQTb340FVals[*usiFOV] & 0X0001) << 5;
  adVALUES[liPointer + 25] = (double)usiWorkBits;
  if (FILL_BYTE_J2 == pscTbInterFVals[*usiFOV])
    adVALUES[liPointer + 26] = (double)BUFR_MISSING;
                               /* Radiometer brightness temperature interpolation flag */
  else
    adVALUES[liPointer + 26] = (double)pscTbInterFVals[*usiFOV];
                               /* Radiometer brightness temperature interpolation flag */
  if (FILL_BYTE_J2 == pscIceVals[*usiFOV]) /* Ice flag */
    adVALUES[liPointer + 27] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 27] = (double)pscIceVals[*usiFOV]; /* Ice flag */

/* AR 160 - Add aux altimeter state flags and reset field indices */
/*                                            (MOD/SSE, 18/01/10) */
  usiWorkBits = 0; /*aux altimeter state flags */
  usiWorkBits |= (pscASFBandSeqVals[*usiFOV] & 0X0001) << 4;
  usiWorkBits |= (pscASFCBandVals[*usiFOV] & 0X0001) << 3;
  usiWorkBits |= (pscASFCBandStatVals[*usiFOV] & 0X0001) << 2;
  usiWorkBits |= (pscASFKuBandStatVals[*usiFOV] & 0X0001) << 1;
  adVALUES[liPointer + 28] = (double)usiWorkBits;
/* AR 160 - Add met map availability (MOD/SSE, 18/01/10) */
  if (FILL_BYTE_J2 == pscECMWFMapAvlVals[*usiFOV])
    adVALUES[liPointer + 29] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 29] = (double)pscECMWFMapAvlVals[*usiFOV];
/* AR 160 - Add interpolation flag for MDT (MOD/SSE, 18/01/10) */
  if (FILL_BYTE_J2 == pscInterpFlagMDTVals[*usiFOV])
    adVALUES[liPointer + 30] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 30] = (double)pscInterpFlagMDTVals[*usiFOV];

/* Set altimeter: Ku band */
/* AR 63 - set fill values in netCDF to missing in BUFR (MOD/SSE, 17/06/08) */
  if (FILL_INT_J2 == piRangeKuVals[*usiFOV])
    adVALUES[liPointer + 31] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 31] = (double)((piRangeKuVals[*usiFOV] / 1.0e4) + 1.3e6);
  if (FILL_SHORT_J2 == psiRMSRanKuVals[*usiFOV])
    adVALUES[liPointer + 32] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 32] = (double)(psiRMSRanKuVals[*usiFOV] / 1.0e4);
  if (FILL_BYTE_J2 == pscNumRanKuVals[*usiFOV])
    adVALUES[liPointer + 33] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 33] = (double)(pscNumRanKuVals[*usiFOV]);
  if (FILL_INT_J2 == piInstCorrKuVals[*usiFOV])
    adVALUES[liPointer + 34] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 34] = (double)(piInstCorrKuVals[*usiFOV] / 1.0e4);
/* AR 136 - The fill value for psiSSBCorrKuVals needs to be explicitly
            dealt with as it lies in the valid range (MOD/SSE, 25/06/09) */
  if ((FILL_SHORT_J2 == psiSSBCorrKuVals[*usiFOV]) || 
    (((double)psiSSBCorrKuVals[*usiFOV] / 1.0e4 < -32.768) ||
    ((double)psiSSBCorrKuVals[*usiFOV] / 1.0e4 > 32.766)))
    adVALUES[liPointer + 35] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 35] = (double)(psiSSBCorrKuVals[*usiFOV] / 1.0e4);
  if (FILL_SHORT_J2 == psiSWHKuVals[*usiFOV])
    adVALUES[liPointer + 36] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 36] = (double)(psiSWHKuVals[*usiFOV] / 1.0e3);
  if (FILL_SHORT_J2 == psiRMSSWHKuVals[*usiFOV])
    adVALUES[liPointer + 37] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 37] = (double)(psiRMSSWHKuVals[*usiFOV] / 1.0e3);
  if (FILL_BYTE_J2 == pscNumSWHKuVals[*usiFOV])
    adVALUES[liPointer + 38] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 38] = (double)(pscNumSWHKuVals[*usiFOV]);
  if (FILL_SHORT_J2 == psiInstCorrSWHKuVals[*usiFOV])
    adVALUES[liPointer + 39] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 39] = (double)(psiInstCorrSWHKuVals[*usiFOV] / 1.0e3);
  if (FILL_SHORT_J2 == psiSig0KuVals[*usiFOV])
    adVALUES[liPointer + 40] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 40] = (double)(psiSig0KuVals[*usiFOV] / 1.0e2);
  if (FILL_SHORT_J2 == psiRMSSig0KuVals[*usiFOV])
    adVALUES[liPointer + 41] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 41] = (double)(psiRMSSig0KuVals[*usiFOV] / 1.0e2);
  if (FILL_BYTE_J2 == pscNumSig0KuVals[*usiFOV])
    adVALUES[liPointer + 42] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 42] = (double)(pscNumSig0KuVals[*usiFOV]);
  if (((double)psiInstCorrSig0KuVals[*usiFOV] / 1.0e2 < -20.48) ||
    ((double)psiInstCorrSig0KuVals[*usiFOV] / 1.0e2 > 20.46))
    adVALUES[liPointer + 43] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 43] = (double)(psiInstCorrSig0KuVals[*usiFOV] / 1.0e2);
  if (FILL_BYTE_J2 == pscAtCorrSig0KuVals[*usiFOV])
    adVALUES[liPointer + 44] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 44] = (double)(pscAtCorrSig0KuVals[*usiFOV] / 1.0e2);
  if (((double)psiAGCKuVals[*usiFOV] / 1.0e2 < 0.0) ||
    ((double)psiAGCKuVals[*usiFOV] / 1.0e2 > 81.9))
    adVALUES[liPointer + 45] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 45] = (double)(psiAGCKuVals[*usiFOV] / 1.0e2);
  if (((double)psiRMSAGCKuVals[*usiFOV] / 1.0e2 < 0.0) ||
    ((double)psiRMSAGCKuVals[*usiFOV] / 1.0e2 > 2.54))
    adVALUES[liPointer + 46] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 46] = (double)(psiRMSAGCKuVals[*usiFOV] / 1.0e2);
  if (FILL_BYTE_J2 == pscNumAGCKuVals[*usiFOV])
    adVALUES[liPointer + 47] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 47] = (double)(pscNumAGCKuVals[*usiFOV]);

/* Set altimeter: C band */
/* AR 63 - set fill values in netCDF to missing in BUFR (MOD/SSE, 16/06/08) */
  if (FILL_INT_J2 == piRangeCVals[*usiFOV])
    adVALUES[liPointer + 48] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 48] = (double)((piRangeCVals[*usiFOV] / 1.0e4) + 1.3e6);
  if (FILL_SHORT_J2 == psiRMSRanCVals[*usiFOV])
    adVALUES[liPointer + 49] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 49] = (double)(psiRMSRanCVals[*usiFOV] / 1.0e4);
  if (FILL_BYTE_J2 == pscNumRanCVals[*usiFOV])
    adVALUES[liPointer + 50] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 50] = (double)(pscNumRanCVals[*usiFOV]);
  if (FILL_INT_J2 == piInstCorrCVals[*usiFOV])
    adVALUES[liPointer + 51] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 51] = (double)(piInstCorrCVals[*usiFOV] / 1.0e4);
  if (((double)psiSSBCorrCVals[*usiFOV] / 1.0e4 < -0.6) ||
    ((double)psiSSBCorrCVals[*usiFOV] / 1.0e4 > 0.219))
    adVALUES[liPointer + 52] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 52] = (double)(psiSSBCorrCVals[*usiFOV] / 1.0e4);
  if (FILL_SHORT_J2 == psiSWHCVals[*usiFOV])
    adVALUES[liPointer + 53] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 53] = (double)(psiSWHCVals[*usiFOV] / 1.0e3);
  if (FILL_SHORT_J2 == psiRMSSWHCVals[*usiFOV])
    adVALUES[liPointer + 54] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 54] = (double)(psiRMSSWHCVals[*usiFOV] / 1.0e3);
  if (FILL_BYTE_J2 == pscNumSWHCVals[*usiFOV])
    adVALUES[liPointer + 55] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 55] = (double)(pscNumSWHCVals[*usiFOV]);
  if (FILL_SHORT_J2 == psiInstCorrSWHCVals[*usiFOV])
    adVALUES[liPointer + 56] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 56] = (double)(psiInstCorrSWHCVals[*usiFOV] / 1.0e3);
  if (FILL_SHORT_J2 == psiSig0CVals[*usiFOV])
    adVALUES[liPointer + 57] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 57] = (double)(psiSig0CVals[*usiFOV] / 1.0e2);
  if (FILL_SHORT_J2 == psiRMSSig0CVals[*usiFOV])
    adVALUES[liPointer + 58] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 58] = (double)(psiRMSSig0CVals[*usiFOV] / 1.0e2);
  if (FILL_BYTE_J2 == pscNumSig0CVals[*usiFOV])
    adVALUES[liPointer + 59] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 59] = (double)(pscNumSig0CVals[*usiFOV]);
  if (((double)psiInstCorrSig0CVals[*usiFOV] / 1.0e2 < -20.48) ||
    ((double)psiInstCorrSig0CVals[*usiFOV] / 1.0e2 > 20.46))
    adVALUES[liPointer + 60] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 60] = (double)(psiInstCorrSig0CVals[*usiFOV] / 1.0e2);
  if (FILL_BYTE_J2 == pscAtCorrSig0CVals[*usiFOV])
    adVALUES[liPointer + 61] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 61] = (double)(pscAtCorrSig0CVals[*usiFOV] / 1.0e2);
  if (((double)psiAGCCVals[*usiFOV] / 1.0e2 < 0.0) ||
    ((double)psiAGCCVals[*usiFOV] / 1.0e2 > 81.9))
    adVALUES[liPointer + 62] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 62] = (double)(psiAGCCVals[*usiFOV] / 1.0e2);
  if (((double)psiRMSAGCCVals[*usiFOV] / 1.0e2 < 0.0) ||
    ((double)psiRMSAGCCVals[*usiFOV] / 1.0e2 > 5.1))
    adVALUES[liPointer + 63] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 63] = (double)(psiRMSAGCCVals[*usiFOV] / 1.0e2);
  if (FILL_BYTE_J2 == pscNumAGCCVals[*usiFOV]) /* Rain flag */
    adVALUES[liPointer + 64] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 64] = (double)(pscNumAGCCVals[*usiFOV]);

/* Set radiometer data */
/* AR 93 - set frequency value to 18.7 GHz (MOD/SSE, 21/08/08) */
/* AR 63 - set fill values in netCDF to missing in BUFR (MOD/SSE, 17/06/08) */
  adVALUES[liPointer + 65] = 1.87E10; /* 18.7 GHz */
  if (FILL_SHORT_J2 == psiTb187Vals[*usiFOV])
    adVALUES[liPointer + 66] = (double)BUFR_MISSING;
/* AR 86 - robustly handle negative temperatures from TM-NRT */
/*         (MOD/SSE, 19/08/08) */
  else if (psiTb187Vals[*usiFOV] < 0)
  {
    adVALUES[liPointer + 66] = (double)BUFR_MISSING;
    if (OUT)(void)printf("tb_187 contains a negative temperature.");
    /* Write syslog message for monitoring */
    if (LOG)
      syslog(LOG_INFO|LOG_LOCAL2, "LOG_INFO: Negative tb_187 value set to MISSING in BUFR data");
  }
  else
    adVALUES[liPointer + 66] = (double)(psiTb187Vals[*usiFOV] / 1.0E2); /* Temperature 187u */
  adVALUES[liPointer + 67] = 2.38E10; /* 23.8 GHz */
  if (FILL_SHORT_J2 == psiTb238Vals[*usiFOV])
    adVALUES[liPointer + 68] = (double)BUFR_MISSING;
/* AR 86 - robustly handle negative temperatures from TM-NRT */
/*         (MOD/SSE, 19/08/08) */
  else if (psiTb238Vals[*usiFOV] < 0)
  {
    adVALUES[liPointer + 68] = (double)BUFR_MISSING;
    if (OUT)(void)printf("tb_238 contains a negative temperature.");
    /* Write syslog message for monitoring */
    if (LOG)
      syslog(LOG_INFO|LOG_LOCAL2, "LOG_INFO: Negative tb_238 value set to MISSING in BUFR data");
  }
  else
    adVALUES[liPointer + 68] = (double)(psiTb238Vals[*usiFOV] / 1.0E2); /* Temperature 238u */
  adVALUES[liPointer + 69] = 3.40E10; /* 34.0 GHz */
  if (FILL_SHORT_J2 == psiTb340Vals[*usiFOV])
    adVALUES[liPointer + 70] = (double)BUFR_MISSING;
/* AR 86 - robustly handle negative temperatures from TM-NRT */
/*         (MOD/SSE, 19/08/08) */
  else if (psiTb340Vals[*usiFOV] < 0)
  {
    adVALUES[liPointer + 70] = (double)BUFR_MISSING;
    if (OUT)(void)printf("tb_340 contains a negative temperature.");
    /* Write syslog message for monitoring */
    if (LOG)
      syslog(LOG_INFO|LOG_LOCAL2, "LOG_INFO: Negative tb_340 value set to MISSING in BUFR data");
  }
  else
    adVALUES[liPointer + 70] = (double)(psiTb340Vals[*usiFOV] / 1.0E2); /* Temperature 340u */
/* AR 97 - Convert values from g/cm^2 (with scale factor 100) to kg/m^-2 */
/*         by mutiplying by 10. (MOD/SSE, 22/09/08) */
/* ECP 5 - Values now (prod spec 2.3) in kg/m^-2 with scale factor of 10 */
/*         (MOD/SSE, 05/11/08) */
  if ((((double)psiRadWVVals[*usiFOV] / 1.0E1) < 0.0) ||
      (((double)psiRadWVVals[*usiFOV] / 1.0E1) > 102.2))
    adVALUES[liPointer + 71] = (double)BUFR_MISSING; /* Radiometer WV content */
  else
    adVALUES[liPointer + 71] = (double)psiRadWVVals[*usiFOV] / 1.0E1;
                                                     /* Radiometer WV content */
  if (((double)psiRadLWVals[*usiFOV] / 1.0E2 < 0.0) ||
    ((double)psiRadLWVals[*usiFOV] / 1.0E2 > 2.54))
    adVALUES[liPointer + 72] = (double)BUFR_MISSING; /* Radiometer  liquid water */
  else
    adVALUES[liPointer + 72] = (double)psiRadLWVals[*usiFOV] / 1.0E2; /* Radiometer liquid water */

/* Set wind data */
/* AR 63 - set fill values in netCDF to missing in BUFR (MOD/SSE, 17/06/08) */
  adVALUES[liPointer + 73] = (double)10.0; /* Height co-ord for wind is 10 m */
  if (((double)psiWSpdAltVals[*usiFOV] / 1.0E2 < 0.0) ||
    ((double)psiWSpdAltVals[*usiFOV] / 1.0E2 > 40.94))
    adVALUES[liPointer + 74] = (double)BUFR_MISSING; /*Wind spd from alt */
  else
    adVALUES[liPointer + 74] = (double)psiWSpdAltVals[*usiFOV] / 1.0E2; /* Wind spd from alt */
  if (((double)psiWSpdRadVals[*usiFOV] / 1.0E2 < 0.0) ||
    ((double)psiWSpdRadVals[*usiFOV] / 1.0E2 > 40.94))
    adVALUES[liPointer + 75] = (double)BUFR_MISSING; /*Wind spd from rad */
  else
    adVALUES[liPointer + 75] = (double)psiWSpdRadVals[*usiFOV] / 1.0E2; /* Wind spd from rad */
  adVALUES[liPointer + 76] = (double)BUFR_MISSING; /* Turn off height co-ordinate */
  if (FILL_SHORT_J2 == psiWSpdUVals[*usiFOV])
    adVALUES[liPointer + 77] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 77] = (double)psiWSpdUVals[*usiFOV] / 1.0E2; /* Wind spd U model */
  if (FILL_SHORT_J2 == psiWSpdVVals[*usiFOV])
    adVALUES[liPointer + 78] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 78] = (double)psiWSpdVVals[*usiFOV] / 1.0E2; /* Wind spd V model */

/* Set dynamic topography */
/* AR 63 - set fill values in netCDF to missing in BUFR (MOD/SSE, 17/06/08) */
/* Mean dynamic topography */
  if (((double)piMeanDynTopVals[*usiFOV] / 1.0e4 < -131.072) ||
    ((double)piMeanDynTopVals[*usiFOV] / 1.0e4 > 131.070))
    adVALUES[liPointer + 79] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 79] = (double)piMeanDynTopVals[*usiFOV] / 1.0e4;
/* Height */
  if (FILL_INT_J2 == piHeightVals[*usiFOV])
    adVALUES[liPointer + 80] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 80] = (double)(piHeightVals[*usiFOV] / 1.0e4) + 1.3e6;
/* Instantaneous altitude rate */
/* AR 131 - Correct the index used for missing values of orb_alt_rate */
/*         (MOD/SSE, 25/06/09) */
  if (FILL_SHORT_J2 == psiInstAltRVals[*usiFOV])
    adVALUES[liPointer + 81] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 81] = (double)psiInstAltRVals[*usiFOV] / 1.0e2;
/* Off nadir angle (platform) */
  if (FILL_SHORT_J2 == psiOffNadPtfVals[*usiFOV])
    adVALUES[liPointer + 82] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 82] = (double)psiOffNadPtfVals[*usiFOV] / 1.0e4;
/* Off nadir angle (waveform) */
/* AR 136 - The fill value for psiOffNadWvVals needs to be explicitly
            dealt with as it lies in the valid range (MOD/SSE, 25/06/09) */
  if ((FILL_SHORT_J2 == psiOffNadWvVals[*usiFOV]) || 
    (((double)psiOffNadWvVals[*usiFOV] / 1.0e4 < -327.68) ||
    ((double)psiOffNadWvVals[*usiFOV] / 1.0e4 > 327.66)))
    adVALUES[liPointer + 83] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 83] = (double)psiOffNadWvVals[*usiFOV] / 1.0e4;
/* Model ionospheric correction on Ku band */
  if (FILL_SHORT_J2 == psiModIonCorVals[*usiFOV])
    adVALUES[liPointer + 84] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 84] = (double)psiModIonCorVals[*usiFOV] / 1.0e4;
/* Altimeter ionospheric correction on Ku band */
  if (FILL_SHORT_J2 == psiAltIonCorVals[*usiFOV])
    adVALUES[liPointer + 85] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 85] = (double)psiAltIonCorVals[*usiFOV] / 1.0e4;
/* Model dry tropospheric correction */
  if (FILL_SHORT_J2 == psiMDryTCorVals[*usiFOV])
    adVALUES[liPointer + 86] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 86] = (double)psiMDryTCorVals[*usiFOV] / 1.0e4;
/* Model wet tropospheric correction */
  if (FILL_SHORT_J2 == psiMWetTCorVals[*usiFOV])
    adVALUES[liPointer + 87] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 87] = (double)psiMWetTCorVals[*usiFOV] / 1.0e4;
/* Radiometer wet tropospheric correction */
  if (((double)psiRadWetTCorVals[*usiFOV] / 1.0e4 < -0.5000) ||
    ((double)psiRadWetTCorVals[*usiFOV] / 1.0e4 > 0.3190))
    adVALUES[liPointer + 88] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 88] = (double)psiRadWetTCorVals[*usiFOV] / 1.0e4;
/* Geophysical parameters */
  if (((double)piMSSVals[*usiFOV] / 1.0e4 < -131.072) ||
    ((double)piMSSVals[*usiFOV] / 1.0e4 > 131.07))
    adVALUES[liPointer + 89] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 89] = (double)piMSSVals[*usiFOV] / 1.0e4;
/* AR 63 - variable mss_tp_along_track not available in J2 O/IGDRs (MOD/SSE, 11/06/08) */
/*
  if (((double)piMSSAltVals[*usiFOV] / 1.0e4 < -131.072) ||
    ((double)piMSSAltVals[*usiFOV] / 1.0e4 > 131.07))
    adVALUES[liPointer + 87] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 87] = (double)piMSSAltVals[*usiFOV] / 1.0e4;
*/
  adVALUES[liPointer + 90] = (double)BUFR_MISSING;
  if (FILL_INT_J2 == piGeoidVals[*usiFOV])
    adVALUES[liPointer + 91] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 91] = (double)piGeoidVals[*usiFOV] / 1.0e4;
  if (FILL_INT_J2 == piBathyVals[*usiFOV])
    adVALUES[liPointer + 92] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 92] = (double)piBathyVals[*usiFOV];
  if (FILL_SHORT_J2 == psiSolidTideVals[*usiFOV])
    adVALUES[liPointer + 93] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 93] = (double)psiSolidTideVals[*usiFOV] / 1.0e4;
  if (((double)piGeoTide1Vals[*usiFOV] / 1.0e4 < -32.768) ||
    ((double)piGeoTide1Vals[*usiFOV] / 1.0e4 > 32.766))
    adVALUES[liPointer + 94] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 94] = (double)piGeoTide1Vals[*usiFOV] / 1.0e4;
  if (((double)piGeoTide2Vals[*usiFOV] / 1.0e4 < -32.768) ||
    ((double)piGeoTide2Vals[*usiFOV] / 1.0e4 > 32.766))
    adVALUES[liPointer + 95] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 95] = (double)piGeoTide2Vals[*usiFOV] / 1.0e4;
  if (FILL_SHORT_J2 == psiLoadTide1Vals[*usiFOV])
    adVALUES[liPointer + 96] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 96] = (double)psiLoadTide1Vals[*usiFOV] / 1.0e4;
  if (FILL_SHORT_J2 == psiLoadTide2Vals[*usiFOV])
    adVALUES[liPointer + 97] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 97] = (double)psiLoadTide2Vals[*usiFOV] / 1.0e4;
  if (FILL_SHORT_J2 == psiOcTideEqVals[*usiFOV])
    adVALUES[liPointer + 98] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 98] = (double)psiOcTideEqVals[*usiFOV] / 1.0e4;
  if (((double)psiOcTideNonEqVals[*usiFOV] / 1.0e4 < -0.2) ||
    ((double)psiOcTideNonEqVals[*usiFOV] / 1.0e4 > 0.2094))
    adVALUES[liPointer + 99] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 99] = (double)psiOcTideNonEqVals[*usiFOV] / 1.0e4;
  if (FILL_SHORT_J2 == psiPoleTideVals[*usiFOV])
    adVALUES[liPointer + 100] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 100] = (double)psiPoleTideVals[*usiFOV] / 1.0e4;
  if (FILL_SHORT_J2 == psiInvBathVals[*usiFOV])
    adVALUES[liPointer + 101] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 101] = (double)psiInvBathVals[*usiFOV] / 1.0e4;
/* AR 134 - Field 99 needs to be populated from hf_fluctuations_corr */
/*          which is in psiHfFlucVals (MOD/SSE, 25/06/09) */
  if (((double)psiHfFlucVals[*usiFOV] / 1.0e4 < -0.3) ||
    ((double)psiHfFlucVals[*usiFOV] / 1.0e4 > 0.519))
    adVALUES[liPointer + 102] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 102] = (double)psiHfFlucVals[*usiFOV] / 1.0e4;
/* AR 160 - Add sea surface height anomoly (MOD/SSE, 18/01/10) */
  if (((double)psiSSHAVals[*usiFOV] / 1.0e3 < -32.768) ||
    ((double)psiSSHAVals[*usiFOV] / 1.0e3 > 32.766))
    adVALUES[liPointer + 103] = (double)BUFR_MISSING;
  else
    adVALUES[liPointer + 103] = (double)psiSSHAVals[*usiFOV] / 1.0e3;
}

/* Unpack lastJASON2 OGDR data from data for encoding */
void vUnPackLastLineJ2OGDR(short int *siPointInBulletin, int32_t *liKELEM,
  double adVALUES[])
{
  long int   liPointer;
  long int   liOffset;

/* Blank out data in BUFR encoding array */
  liPointer = *siPointInBulletin * (*liKELEM);
  for (liOffset = 0; liOffset < *liKELEM; liOffset++)
    adVALUES[liPointer + liOffset] = BUFR_MISSING;
}

/* Use satellite ID to switch T2 in bulletin header */
/* Write one bulletin into an RMDCN type file */
void vWriteBulletin(int32_t aliKBUFF[], uint32_t *uliTotBUFRLength,
  char *pzProcCen, short int *siSeqNumber, unsigned short int *usiDay,
  unsigned short int *usiHour, unsigned short int *usiMinute,
  unsigned short int *usiSpaceCraftID, FILE *fpOutFile)
{
  char azABH[19];
  static unsigned short int usiPrevDay;
  static unsigned short int usiPrevHour;
  static unsigned short int usiPrevMinute;
  unsigned long int uliWord;

/* If already used this date and time, update sequence number,
     otherwise rest it to 1 */
  if ((*usiMinute == usiPrevMinute) && (*usiHour == usiPrevHour) &&
    (*usiDay == usiPrevDay))
  {
    *siSeqNumber += 1;
  }
  else *siSeqNumber = 1;

/* create ABH */
/* ECP 5 - Global attribute processing_center has new values in v2.3 of prod specn */
/*         (MOD/SSE, 25/11/08) */
  if (strstr(pzProcCen, "EUM") != NULL) /* EUMETSAT */
    sprintf(azABH, "ISZX%2.2d EUMS %2.2d%2.2d%2.2d",
      *siSeqNumber, *usiDay, *usiHour, *usiMinute);
  else if (strstr(pzProcCen, "NOAA") != NULL) /* NOAA */
    sprintf(azABH, "ISZX%2.2d KNES %2.2d%2.2d%2.2d",
      *siSeqNumber, *usiDay, *usiHour, *usiMinute);
  else if (strstr(pzProcCen, "ESPC") != NULL) /* NOAA */
    sprintf(azABH, "ISZX%2.2d KNES %2.2d%2.2d%2.2d",
      *siSeqNumber, *usiDay, *usiHour, *usiMinute);
  else if (strstr(pzProcCen, "CNES") != NULL) /* CNES */
    sprintf(azABH, "ISZX%2.2d LFPW %2.2d%2.2d%2.2d",
      *siSeqNumber, *usiDay, *usiHour, *usiMinute);
  else if (strstr(pzProcCen, "SALP") != NULL) /* CNES */
    sprintf(azABH, "ISZX%2.2d LFPW %2.2d%2.2d%2.2d",
      *siSeqNumber, *usiDay, *usiHour, *usiMinute);
  else
    sprintf(azABH, "ISZX%2.2d EUMS %2.2d%2.2d%2.2d",
      *siSeqNumber, *usiDay, *usiHour, *usiMinute);

/* Swap output values if required (look for BUFR in first 4 bytes) */
  if (0X42554652L == aliKBUFF[0])
    for (uliWord = 0; uliWord <= (800000); uliWord++)
      aliKBUFF[uliWord] = htonl(aliKBUFF[uliWord]);

/* write data to file */
  (void)fprintf(fpOutFile, "%8.8d00\1\r\r\n000\r\r\n%s\r\r\n",
    *uliTotBUFRLength + 35, azABH);

  if (1 != fwrite(aliKBUFF, (size_t)*uliTotBUFRLength, 1, fpOutFile))
  {
    if (OUT)
      (void)printf("Could not append bulletin %s data to output file.\n", azABH);
  }

  (void)fprintf(fpOutFile, "\r\r\n\3");

/* Reset previous time information */
  usiPrevDay = *usiDay;
  usiPrevHour = *usiHour;
  usiPrevMinute = *usiMinute;
}

void vOGDRSecstoYMDHMS(long int liOGDRSecs,
  unsigned short int *usiYear, unsigned short int *usiMonth,
  unsigned short int *usiDay, unsigned short int *usiHour,
  unsigned short int *usiMinute, unsigned short int *usiSec)
{

  time_t lSecstime; /* Number of seconds since satart of J2 epoch */
  time_t lEpochStart; /* Number of seconds to start of J2 epoch */
  struct tm stDateTime;    /* Local G_TD_TIME structure    */ 

/* Convert integer seconds since J2 epoch start to time_t */
  lSecstime = (time_t)liOGDRSecs;

/* Epoch start time is the number of seconds between the start    */
/* of the UNIX epoch (Jan  1 00:00:00 UTC 1970) and the start     */
/* of the J2 epoch (Jan  1 00:00:00 UTC 2000).  This value can be */
/* On a NOAA-like linux host, this value can be found with        */
/* > date -u -d "Jan 01 00:00:00 GMT 2000" +%s                    */
/* and is 946684800. This can be checked with                     */
/* > date -u -d @946684800                                        */
/* which should give                                              */
/* Sat Jan  1 00:00:00 UTC 2000                                   */
  lEpochStart = 946684800;

/* Calculate total seconds since start of 1970 */
  lSecstime += lEpochStart;

/* convert seconds into date time */
  stDateTime = *gmtime(&lSecstime);

/* BUFR month is 1  12 rather than 0  11 */
  *usiYear = stDateTime.tm_year + 1900;
  *usiMonth = stDateTime.tm_mon + 1;
  *usiDay = stDateTime.tm_mday;
  *usiHour = stDateTime.tm_hour;
  *usiMinute = stDateTime.tm_min;
  *usiSec = stDateTime.tm_sec;
}
