#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <syslog.h>
#include <errno.h>
#include <netinet/in.h>
#include <inttypes.h>
#include <netcdf.h>
#include <unistd.h>
#define SAT_ID 261 /* JASON 2 */
#define INST_ID 366 /* JMR */
#define MAX_OUTPUT_FILENAME_LENGTH 256

/* Updated for AR 63 (handle all fill values in netCDF) (MOD/SSE, 16/06/08) */
#define FILL_BYTE_J2  0X007F
#define FILL_SHORT_J2 0X7FFF
#define FILL_INT_J2   0X7FFFFFFFL

/* FIST version */
/* Updated for AR 52 (datation in Section 1) (MOD/SSE, 28/04/08) */
/* #define FIST_VERS 1.2 */
/* Updated for AR 63 (align to product format specn v2.1) (MOD/SSE, 11/06/08) */
/* #define FIST_VERS 1.3 */
/* Updated for AR 86 (robust handling of negative temperatures) (MOD/SSE, 19/08/08) */
/* #define FIST_VERS 1.4 */
/* Updated for AR 93 (frequency wrongly set for 18.7 GHz) */
/* Updated for AR 97 (units not SI for radiometer liquid water content) */
/* Updated for ECP 5 (align to product format specn v2.3) */
/* Updated for AR 102 (longitude range incorrect) */
/* Updated for AR 103 (some flags set wrongly due to bit ordering) (MOD/SSE, 13/11/08) */
/* #define FIST_VERS 1.5 */
/* Updated for ECP 5 (allow for new global attribute values) (MOD/SSE, 26/11/08) */
/* #define FIST_VERS 1.6 */
/* Updated for AR 130 (GTS file names and ISO 3166) (MOD/SSE, 25/06/09) */
/* Updated for AR 131 (index for orb_alr_rate wrong) (MOD/SSE, 25/06/09) */
/* Updated for AR 134 (index wrong for hf_fluctuations_corr) (MOD/SSE, 25/06/09) */
/* Updated for AR 136 (fill values not corectly handled) (MOD/SSE, 25/06/09) */
/* Updated for AR 130 (GTS file names and ISO 3166) (MOD/SSE, 18/01/10) */
/* Updated for AR 160 (Addition of missed variables) (MOD/SSE, 18/01/10) */
#define FIST_VERS 1.8

/* ECMWF library version */
/* Updated for AR 143 (New version of ECMWF library) (MOD/SSE, 01/02/10) */
#define ECMWF_LIB_VERS 380

/* Handle errors by printing an error message and exiting with a
 * non-zero status. */
#define ERRCODE 2
#define ERR(e) {printf("Error: %s\n", nc_strerror(e)); exit(ERRCODE);}

#pragma pack(1)

/* Set to 1 for print out, set to 0 for silent */
#define OUT 0

/* Set to 1 for syslog messages, set to 0 for none */
#define LOG 1

/* BUFR encoding parameters */
/* AR 160 - Use MT Version 14 (MOD/SSE, 18/01/10) */
#define FOV_PER_BULLETIN 750
#define BUFR_EDITION 4
#define BUFR_MT_VERSION 14 /* Descriptors used are effectively added to v14 */
#define BUFR_LT_VERSION 0 /* Descriptors used are not from local range */
#define ORIG_CENTRE_EUM 254
#define ORIG_CENTRE_NOAA 160
#define ORIG_CENTRE_CNES 84
#define ORIG_SUB_CENTRE 0
#define BUFR_MISSING 1.7E38
#define bufren bufren_

/* BUFR Variables */
  int32_t   aliKSEC0[3];
  int32_t   aliKSEC1[40];
  int32_t   aliKSEC2[4096];
  int32_t   aliKSEC3[4];
  int32_t   aliKSEC4[2];
  int32_t   liKTDLEN;
  int32_t   aliKTDLST[180000];
  int32_t   liKDLEN;
  int32_t   aliKDATA[10];
  int32_t   liKELEM;
  int32_t   liKVALS;
  double     adVALUES[2500000];
  char       azCVALS[95][80];
  int32_t   liKBUFL;
  int32_t   aliKBUFF[800000];
  int32_t   liKERR;

short int siCommandLineOptions(int *argc, char *argv[]);

short int siParseInputFileName(int argc, char *argv[], char azJ2FileName[],
  char azProdVers[], char azCycle[], short int *siCycle,
  char azPass[], short int *siPass, char azStopTime[]);

void vBUFREncPrep(char *pzProcCen, char *pzCreatTime,
  int32_t aliKSEC0[], int32_t aliKSEC1[],
  int32_t aliKSEC2[], int32_t aliKSEC3[], int32_t aliKSEC4[],
  int32_t *liKTDLEN, int32_t aliKTDLST[], int32_t *liKDLEN,
  int32_t *liKELEM, int32_t *liKVALS, double adVALUES[]);

/* AR 63 - Add radiometer state flag to argument list (MOD/SSE, 11/06/08) */
/* AR 160 - Pack additional parameters (MOD/SSE, 18/01/10) */
void vPackLineJ2OGDR(short int *siPointInBulletin,
  unsigned short int *usiFOV, int *iCycleNo, int *iAbsRevNo,
  short int *psiTb187Vals, short int *psiTb238Vals, short int *psiTb340Vals,
  double *pdTimeVals, int *piLatVals, int *piLonVals,
  signed char *pscSurfVals, signed char *pscAltEchoVals, signed char *pscRadSurfVals,
  short int *psiWSpdUVals, short int *psiWSpdVVals, short int *psiWSpdAltVals,
  short int *psiWSpdRadVals, short int *psiRadWVVals, short int *psiRadLWVals,
  int *piMeanDynTopVals, int *piHeightVals, short int *psiInstAltRVals,
  short int *psiOffNadPtfVals, short int *psiOffNadWvVals, short int *psiModIonCorVals,
  short int *psiAltIonCorVals, short int *psiMDryTCorVals, short int *psiMWetTCorVals,
  short int *psiRadWetTCorVals, signed char *pscIntMSSVals,
  signed char *pscTSol1FVals, signed char *pscTSol2FVals, signed char *pscInterMetFVals,
  signed char *pscOrbStatFVals, signed char *pscRadStatFVals,
  signed char *pscAltStatFVals, signed char *pscAltKuRFVals,
  signed char *pscAltCRFVals, signed char *pscAltKuSWHFVals, signed char *pscAltCSWHFVals,
  signed char *pscAltKuSigFVals, signed char *pscAltCSigFVals, signed char *pscOffNadKuFVals,
  signed char *pscOffNafPtfFVals, signed char *pscCorKuRFVals, signed char *pscCorCRFVals,
  signed char *pscCorKuSWHFVals, signed char *pscCorCSWHFVals, signed char *pscCorKuSigFVals,
  signed char *pscCorCSigFVals, signed char *pscRainFVals, signed char *pscQTb187FVals,
  signed char *pscQTb238FVals, signed char *pscQTb340FVals, signed char *pscTbInterFVals,
  signed char *pscIceVals, int *piRangeKuVals, short int *psiRMSRanKuVals,
  signed char *pscNumRanKuVals, int *piInstCorrKuVals, short int *psiSSBCorrKuVals,
  short int *psiSWHKuVals, short int *psiRMSSWHKuVals, signed char *pscNumSWHKuVals,
  short int *psiInstCorrSWHKuVals, short int *psiSig0KuVals, short int *psiRMSSig0KuVals,
  signed char *pscNumSig0KuVals, short int *psiInstCorrSig0KuVals,
  signed char *pscAtCorrSig0KuVals, short int *psiAGCKuVals, short int *psiRMSAGCKuVals,
  signed char *pscNumAGCKuVals, int *piRangeCVals, short int *psiRMSRanCVals,
  signed char *pscNumRanCVals, int *piInstCorrCVals, short int *psiSSBCorrCVals,
  short int *psiSWHCVals, short int *psiRMSSWHCVals, signed char *pscNumSWHCVals,
  short int *psiInstCorrSWHCVals, short int *psiSig0CVals, short int *psiRMSSig0CVals,
  signed char *pscNumSig0CVals, short int *psiInstCorrSig0CVals,
  signed char *pscAtCorrSig0CVals, short int *psiAGCCVals, short int *psiRMSAGCCVals,
  signed char *pscNumAGCCVals, int *piMSSVals, int *piMSSAltVals, int *piGeoidVals,
  int *piBathyVals, short int *psiSolidTideVals, int *piGeoTide1Vals, int *piGeoTide2Vals,
  short int *psiLoadTide1Vals, short int *psiLoadTide2Vals, short int *psiOcTideEqVals,
  short int *psiOcTideNonEqVals, short int *psiPoleTideVals, short int *psiInvBathVals,
  short int *psiHfFlucVals, signed char *pscASFCBandVals, signed char *pscASFCBandStatVals,
  signed char *pscASFKuBandStatVals, signed char *pscASFBandSeqVals,
  signed char *pscECMWFMapAvlVals, signed char *pscInterpFlagMDTVals,
  short int *psiSSHAVals, int32_t *liKELEM, double adVALUES[]);

void vUnPackLastLineJ2OGDR(short int *siLineInBulletin, int32_t *liKELEM,
  double adVALUES[]);

void vWriteBulletin(int32_t aliKBUFF[], uint32_t *uliTotBUFRLength,
  char *pzProcCen, short int *siSeqNumber, unsigned short int *usiDay,
  unsigned short int *usiHour, unsigned short int *usiMinute,
  unsigned short int *usiSpaceCraftID, FILE *fpOutFile);

void vOGDRSecstoYMDHMS(long int liOGDRSecs,
  unsigned short int *usiYear, unsigned short int *usiMonth,
  unsigned short int *usiDay, unsigned short int *usiHour,
  unsigned short int *usiMinute, unsigned short int *usiSec);
